@@ -24,19 +24,6 @@
 #include "../config.h"
 #include "rxvt.h"
 
-#ifdef DEBUG_VERBOSE
-#define DEBUG_LEVEL 1
-#else 
-#define DEBUG_LEVEL 0
-#endif
-
-
-#if DEBUG_LEVEL
-#define DBG_MSG(d,x) if(d <= DEBUG_LEVEL) fprintf x
-#else
-#define DBG_MSG(d,x)
-#endif
-
 
 /*
  * Must sync these to macroFnNames in rxvtlib.h.
@@ -57,11 +44,15 @@ static const char *const macroNames[] =
     "ToggleSubwin",	    /* Toggle subwindows (scroll / menu / tabbar) */
     "ResizeFont",	    /* Resize terminal font */
     "ToggleVeryBold",	    /* Toggle use of bold font for colored text */
+    "ToggleBoldColors",	    /* Toggle option boldColors */
+    "ToggleVeryBright",	    /* Toggle option veryBright */
     "ToggleTransparency",   /* Toggle pseudo transparency */
     "ToggleBroadcast",	    /* Toggle broadcasting of input */
     "ToggleHold",	    /* Toggle holding of completed tabs */
     "ToggleFullscreen",	    /* Toggle full screen mode */
+    "Raise",		    /* Raise the terminal window */
     "SetTitle",		    /* Set title of active tab to selection */
+    "UseFifo",		    /* Enable / disable using fifo */
     "PrintScreen",	    /* Dump screen to file / printer */
     "SaveConfig",	    /* Save config to file */
     "ToggleMacros"	    /* Toggle using keyboard macros */
@@ -84,18 +75,6 @@ unsigned char	macro_set_number    ( unsigned char, unsigned char);
  * without causing code bloat. The idea is that defining "macros" can also
  * enable the user to communicate with mrxvt using escape sequences!
  */
-/* {{{ Macro action functions (called from rxvt_dispatch_action) */
-/* EXTPROTO */
-void
-rxvt_toggle_verybold( rxvt_t *r )
-{
-    if (ISSET_OPTION(r, Opt2_veryBold))
-	UNSET_OPTION(r, Opt2_veryBold);
-    else
-	SET_OPTION(r, Opt2_veryBold);
-
-    rxvt_scr_touch (r, ATAB(r), True);
-}
 
 /*
  * str = [+-][s|t|m|b]
@@ -315,7 +294,7 @@ rxvt_parse_macros( rxvt_t *r, const char *str, const char *arg, Bool noReplace)
     newarg[NEWARGLIM - 1] = '\0';
     rxvt_str_trim( newarg );
 
-    DBG_MSG( 1, ( stderr, "Got macro '%s' -- '%s'\n", keyname, newarg ) );
+    rxvt_msg (DBG_INFO, DBG_MACROS, "Got macro '%s' -- '%s'\n", keyname, newarg );
 
     /*
      * Breakup keyname into a keysym and modifier flags.
@@ -358,7 +337,7 @@ rxvt_parse_macros( rxvt_t *r, const char *str, const char *arg, Bool noReplace)
 
     if (NOT_KEYSYM(keysym))
     {
-	rxvt_print_error( "Invalid keysym %s. Skipping macro.",
+	rxvt_msg (DBG_ERROR, DBG_MACROS,  "Invalid keysym %s. Skipping macro.",
 		keyname_nomods);
 	return -1;
     }
@@ -393,8 +372,7 @@ rxvt_add_macro( rxvt_t *r, KeySym keysym, unsigned char modFlags, char *astring,
     action_t	    action;
 
 
-    DBG_MSG( 2, ( stderr, "rxvt_add_macro(%08lx, %2hhx, %s)\n",
-		keysym, modFlags, astring));
+    rxvt_dbgmsg ((DBG_DEBUG, DBG_MACROS, "rxvt_add_macro(%08lx, %2hhx, %s)\n", keysym, modFlags, astring));
 
     /*
      * Check to see if macro already exists.
@@ -421,7 +399,7 @@ rxvt_add_macro( rxvt_t *r, KeySym keysym, unsigned char modFlags, char *astring,
 
 		if( macroNum == MACRO_MAX_CHAINLEN )
 		{
-		    rxvt_print_error( "Macro chain too long" );
+		    rxvt_msg (DBG_ERROR, DBG_MACROS,  "Macro chain too long" );
 		    return 0;
 		}
 
@@ -475,7 +453,7 @@ rxvt_add_macro( rxvt_t *r, KeySym keysym, unsigned char modFlags, char *astring,
     {
 	if( replaceIndex == r->nmacros )
 	{
-	    rxvt_print_error( "No previous macro to add to (key %s%s%s%s)",
+	    rxvt_msg (DBG_ERROR, DBG_MACROS,  "No previous macro to add to (key %s%s%s%s)",
 		    (modFlags & MACRO_CTRL) ? "Ctrl+" : "",
 		    (modFlags & MACRO_META) ? "Meta+" : "",
 		    (modFlags & MACRO_SHIFT) ? "Shift+" : "",
@@ -485,7 +463,7 @@ rxvt_add_macro( rxvt_t *r, KeySym keysym, unsigned char modFlags, char *astring,
 	else if( r->macros[replaceIndex].action.type == MacroFnDummy )
 	{
 	    /* Do not add to a dummy macro */
-	    rxvt_print_error( "Can not add actions to a Dummy macro" );
+	    rxvt_msg (DBG_ERROR, DBG_MACROS,  "Can not add actions to a Dummy macro" );
 	    return 0;	/* Failure */
 	}
 
@@ -552,13 +530,7 @@ rxvt_add_macro( rxvt_t *r, KeySym keysym, unsigned char modFlags, char *astring,
     r->macros[replaceIndex].modFlags	= modFlags;
     r->macros[replaceIndex].action	= action;
 
-    DBG_MSG( 2, ( stderr, "Added macro %hu of %hu. "
-		    "Type %s, len %hu, args '%s'.\n",
-		replaceIndex, r->maxMacros,
-		macroNames[ action.type ], action.len,
-		(action.type == MacroFnStr || action.type == MacroFnEsc) ?
-		    "(escaped string)" :
-		    (IS_NULL(action.str) ? "(nil)" : (char*) action.str)));
+    rxvt_dbgmsg ((DBG_DEBUG, DBG_MACROS, "Added macro %hu of %hu. Type %s, len %hu, args '%s'.\n", replaceIndex, r->maxMacros, macroNames[ action.type ], action.len, (action.type == MacroFnStr || action.type == MacroFnEsc) ?  "(escaped string)" : (IS_NULL(action.str) ? "(nil)" : (char*) action.str)));
 
     return 1;	/* Success */
 }
@@ -623,21 +595,7 @@ rxvt_cleanup_macros( rxvt_t *r )
 	r->maxMacros = r->nmacros;
     }
 
-    DBG_MSG( 3, ( stderr, "Read %d macros. (Have space for %d macros)\n",
-		r->nmacros, r->maxMacros));
-
-#if 0	/* {{{ Debug info */
-    for( i=0; i < r->nmacros; i++)
-    {
-	fprintf( stderr, "%2d. Key 0x%08lx, Mods 0x%1hhx, Chain# 0x%1hhx,"
-		" Type %s %s\n",
-		i, r->macros[i].keysym, r->macros[i].modFlags & MACRO_MODMASK,
-		MACRO_GET_NUMBER( r->macros[i].modFlags ),
-		macroNames[r->macros[i].action.type],
-		r->macros[i].action.str && *r->macros[i].action.str != '\e' ?
-		    r->macros[i].action.str : NULL );
-    }
-#endif	/* }}} */
+    rxvt_dbgmsg ((DBG_DEBUG, DBG_MACROS, "Read %d macros. (Have space for %d macros)\n", r->nmacros, r->maxMacros));
 }
 
 /* {{{1 rxvt_set_action( action, astring)
@@ -654,7 +612,7 @@ rxvt_set_action	    (action_t *action, char *astring)
 {
     unsigned short type, len;
 
-    DBG_MSG( 2, ( stderr, "Setting action '%s'\n", astring));
+    rxvt_dbgmsg ((DBG_DEBUG, DBG_MACROS, "Setting action '%s'\n", astring));
     /*
      * Match head of "astring" to a name in macroNames to figure out the macro
      * type.
@@ -679,7 +637,7 @@ rxvt_set_action	    (action_t *action, char *astring)
 
     if( type == NMACRO_FUNCS )
     {
-	rxvt_print_error( "Action %s is not of known type", astring);
+	rxvt_msg (DBG_ERROR, DBG_MACROS,  "Action %s is not of known type", astring);
 	return False; /* Failure: No matching macro name */
     }
 
@@ -776,8 +734,7 @@ rxvt_process_macros( rxvt_t *r, KeySym keysym, XKeyEvent *ev)
 
     do
       {
-	DBG_MSG( 3, ( stderr, "Processing macro #%d mods %02hhx\n",
-		    macro - r->macros, macro->modFlags ) );
+	rxvt_dbgmsg ((DBG_DEBUG, DBG_MACROS, "Processing macro #%d mods %02hhx\n", macro - r->macros, macro->modFlags));
 	status = rxvt_dispatch_action( r, &(macro->action), (XEvent*) ev );
       }
     while(
@@ -826,7 +783,7 @@ rxvt_dispatch_action( rxvt_t *r, action_t *action, XEvent *ev)
 		rxvt_cmd_write( r, ATAB(r), (unsigned char*) astr, alen - 1);
 	    else
 	    {
-		rxvt_print_error( "Macro %s requires argument.",
+		rxvt_msg (DBG_ERROR, DBG_MACROS,  "Macro %s requires argument.",
 			macroNames[action->type] );
 		retval = -1;
 	    }
@@ -838,7 +795,7 @@ rxvt_dispatch_action( rxvt_t *r, action_t *action, XEvent *ev)
 		rxvt_tt_write( r, ATAB(r), (unsigned char*) astr, alen - 1);
 	    else
 	    {
-		rxvt_print_error( "Macro %s requires argument.",
+		rxvt_msg (DBG_ERROR, DBG_MACROS,  "Macro %s requires argument.",
 			macroNames[action->type] );
 		retval = -1;
 	    }
@@ -912,7 +869,7 @@ rxvt_dispatch_action( rxvt_t *r, action_t *action, XEvent *ev)
 
 	    else
 	    {
-		rxvt_print_error( "Macro %s requires argument.",
+		rxvt_msg (DBG_ERROR, DBG_MACROS,  "Macro %s requires argument.",
 			macroNames[action->type] );
 		retval = -1;
 	    }
@@ -1010,7 +967,7 @@ rxvt_dispatch_action( rxvt_t *r, action_t *action, XEvent *ev)
 		int		amount	    = abs( atoi( (char*) astr ));
 		enum page_dirn	direction   = (*(astr) == '-' ? UP : DN);
 
-		DBG_MSG( 2, ( stderr, "astr: '%s', alen: %d\n", astr, alen ) );
+		rxvt_dbgmsg ((DBG_DEBUG, DBG_MACROS, "astr: '%s', alen: %d\n", astr, alen));
 
 		if( tolower( astr[ alen - 2] ) == 'p' )
 		    /* scroll pages */
@@ -1062,7 +1019,21 @@ rxvt_dispatch_action( rxvt_t *r, action_t *action, XEvent *ev)
 	}
 
 	case MacroFnToggleVeryBold:
-	    rxvt_toggle_verybold(r);
+	    TOGGLE_OPTION( r, Opt2_veryBold );
+
+	    rxvt_scr_touch (r, ATAB(r), True);
+	    break;
+
+	case MacroFnToggleBoldColors:
+	    TOGGLE_OPTION( r, Opt2_boldColors );
+
+	    rxvt_scr_touch (r, ATAB(r), True);
+	    break;
+
+	case MacroFnToggleVeryBright:
+	    TOGGLE_OPTION( r, Opt_veryBright );
+
+	    rxvt_scr_touch (r, ATAB(r), True);
 	    break;
 
 	case MacroFnToggleTransp:
@@ -1100,7 +1071,7 @@ rxvt_dispatch_action( rxvt_t *r, action_t *action, XEvent *ev)
 			break;
 
 		    default:
-			rxvt_print_error( "Badly formed argument '%s' to %s\n",
+			rxvt_msg (DBG_ERROR, DBG_MACROS,  "Badly formed argument '%s' to %s\n",
 				astr, macroNames[action->type] );
 			retval = -1;
 			break;
@@ -1129,7 +1100,7 @@ rxvt_dispatch_action( rxvt_t *r, action_t *action, XEvent *ev)
 
 	    break;
 
-	case MacroFnToggleFullscren:
+	case MacroFnToggleFullscreen:
 	    ewmh_message( r->Xdisplay, XROOT, r->TermWin.parent,
 		XInternAtom( r->Xdisplay, "_NET_WM_STATE", True),
 		_NET_WM_STATE_TOGGLE,
@@ -1137,6 +1108,13 @@ rxvt_dispatch_action( rxvt_t *r, action_t *action, XEvent *ev)
 		0, 0, 0 );
 	    break;
 
+	case MacroFnRaise:
+	    ewmh_message( r->Xdisplay, XROOT, r->TermWin.parent,
+		XInternAtom( r->Xdisplay, "_NET_ACTIVE_WINDOW", True),
+		1, 0 /*timestamp?*/, r->TermWin.parent, 0, 0 );
+	    break;
+
+
 	case MacroFnSetTitle:
 	    if (NOT_NULL(astr))
 		rxvt_tabbar_set_title( r, ATAB(r),
@@ -1148,6 +1126,38 @@ rxvt_dispatch_action( rxvt_t *r, action_t *action, XEvent *ev)
 		retval = -1;
 	    break;
 
+#ifdef USE_FIFO
+	case MacroFnUseFifo:
+	    if( NOT_NULL( astr ) )
+	    {
+		int state = strtol( astr, NULL, 0 );
+
+		if( state == -1 )
+		    state = ISSET_OPTION( r, Opt_useFifo ) ? 0 : 1;
+
+		if( state == 1 && !ISSET_OPTION( r, Opt_useFifo ) )
+		{
+		    SET_OPTION( r, Opt_useFifo );
+		    rxvt_init_fifo( r );
+		}
+
+		else if( state == 0 && ISSET_OPTION( r, Opt_useFifo ) )
+		{
+		    if( r->fifo_fd != -1 )
+		    {
+			close( r->fifo_fd );
+			if( r->num_fds == r->fifo_fd + 1)
+			    rxvt_adjust_fd_number( r );
+
+			r->fifo_fd = -1;
+			unlink( r->fifo_name );
+		    }
+		    UNSET_OPTION( r, Opt_useFifo );
+		}
+	    }
+	    break;
+#endif/*USE_FIFO*/
+
 	case MacroFnPrintScreen:
 	{
 	    /*
@@ -1175,7 +1185,7 @@ rxvt_dispatch_action( rxvt_t *r, action_t *action, XEvent *ev)
 			case 'p': pretty	= 1; break;
 			case 'n': linecont	= 0; break;
 			default:
-			    rxvt_print_error( "Bad option %c to macro %s",
+			    rxvt_msg (DBG_ERROR, DBG_MACROS,  "Bad option %c to macro %s",
 				    *s, macroNames[action->type] );
 			    retval = -1;
 		    }
@@ -1218,7 +1228,7 @@ rxvt_dispatch_action( rxvt_t *r, action_t *action, XEvent *ev)
 	default:
 	    assert( action->type < sizeof( macroNames ) / sizeof( char ** ) );
 
-	    rxvt_print_error( "Macro type '%s' not implemented yet",
+	    rxvt_msg (DBG_ERROR, DBG_MACROS,  "Support for macro type '%s' not compiled.",
 		    macroNames[action->type]);
 	    retval = -1;
     }
