@@ -39,20 +39,6 @@
 
 
 
-#ifdef DEBUG_VERBOSE
-# define DEBUG_LEVEL 1
-# define DEBUG_X
-#else 
-# define DEBUG_LEVEL 0
-#endif
-
-#if DEBUG_LEVEL
-# define DBG_MSG(d,x) if(d <= DEBUG_LEVEL) fprintf x
-#else
-# define DBG_MSG(d,x)
-#endif
-
-
 /*--------------------------------------------------------------------*
  *         BEGIN `INTERNAL' ROUTINE PROTOTYPES                        *
  *--------------------------------------------------------------------*/
@@ -130,7 +116,7 @@ rxvt_pre_show_init( rxvt_t *r )
 
     if( r->TermWin.fade )
     {
-	DBG_MSG( 5, ( stderr, "Allocating space for fade colors\n" ) );
+	rxvt_dbgmsg ((DBG_DEBUG, DBG_MAIN, "Allocating space for fade colors\n"));
 	r->pixColorsUnfocus =
 	    rxvt_malloc( sizeof(unsigned long) * (TOTAL_COLORS) );
 
@@ -170,7 +156,7 @@ rxvt_init(int argc, const char *const *argv)
     /* Initialize vars in "r" */
     if (rxvt_init_vars(r) < 0)
     {
-	rxvt_print_error( "Could not initialize." );
+	rxvt_msg (DBG_ERROR, DBG_MAIN,  "Could not initialize." );
 	rxvt_free(r);
 	return NULL;
     }
@@ -202,7 +188,7 @@ rxvt_init(int argc, const char *const *argv)
 #endif
 
     rxvt_init_env(r);
-    rxvt_init_command(r, cmd_argv);
+    rxvt_init_command(r);
     rxvt_init_screen (r);
 
     /*
@@ -228,7 +214,7 @@ rxvt_init(int argc, const char *const *argv)
     /* Backward compatibility: Open profiles 0 .. n-1 if tnum=n. */
     else if( r->h->rs[Rs_init_term_num] )
     {
-	rxvt_print_error( "Option tnum is obsolete."
+	rxvt_msg (DBG_ERROR, DBG_MAIN,  "Option tnum is obsolete."
 		" Use --initProfileList instead" );
 
 	itnum = atoi( r->h->rs[Rs_init_term_num] );
@@ -270,12 +256,12 @@ rxvt_Child_signal(int sig __attribute__((unused)))
      */
     rxvt_t  *r;
 
-    DBG_MSG( 1, ( stderr, "\e[31mrxvt_Child_signal()\e[0m\n"));
+    rxvt_dbgmsg ((DBG_VERBOSE, DBG_MAIN, "\e[31mrxvt_Child_signal()\e[0m\n"));
 
     r = rxvt_get_r();
     r->ndead_childs++;
 
-    DBG_MSG( 1, ( stderr, "done rxvt_Child_signal()\n"));
+    rxvt_dbgmsg ((DBG_VERBOSE, DBG_MAIN, "done rxvt_Child_signal()\n"));
 }
 
 
@@ -291,7 +277,7 @@ rxvt_Exit_signal(int sig)
 #endif
     rxvt_t*	    r;
 
-    DBG_MSG( 1, ( stderr, "Received signal %d\n", (int) sig));
+    rxvt_msg (DBG_INFO, DBG_MAIN, "Received signal %d\n", (int) sig);
 
     r = rxvt_get_r();
 
@@ -318,13 +304,14 @@ rxvt_alarm_signal( __attribute__((unused)) int sig )
 
     if( LTAB(r) >= 0 )
     {
-	fprintf( stderr, APL_NAME ": WARNING Processes ");
+	rxvt_msg (DBG_WARN, DBG_MAIN, APL_NAME ": WARNING Processes ");
 	for( i=0; i <= LTAB(r); i ++ )
-	    fprintf( stderr, "%d%c", PVTS(r, i)->cmd_pid,
+	    rxvt_msg (DBG_WARN, DBG_MAIN, "%d%c", PVTS(r, i)->cmd_pid,
 		    i == LTAB(r) ? ' ' : ',' );
-	fprintf( stderr, " have not responded to SIGHUP, and are still "
-		"running. Either 'kill -9' these processes or close the "
-		APL_NAME " window again within 3 seconds.\n" );
+	rxvt_msg (DBG_WARN, DBG_MAIN,
+	    " have not responded to SIGHUP, and are still "
+	    "running. Either 'kill -9' these processes or close the "
+	    APL_NAME " window again within 3 seconds.\n");
     }
 }
 
@@ -575,6 +562,17 @@ rxvt_clean_exit (rxvt_t* r)
     XCloseDisplay (r->Xdisplay);
     SET_NULL(r->Xdisplay);
 
+#ifdef USE_FIFO
+    if( r->fifo_fd != -1 )
+    {
+	close( r->fifo_fd );
+	r->fifo_fd = -1;
+
+	unlink( r->fifo_name );
+	rxvt_free( r->fifo_name );  SET_NULL( r->fifo_name );
+    }
+#endif/*USE_FIFO*/
+
     rxvt_free (r->tabstop);		    SET_NULL(r->tabstop);
     rxvt_free (r->pixColorsFocus);	    SET_NULL(r->pixColorsFocus);
 
@@ -673,7 +671,7 @@ rxvt_privileges(int mode)
 void
 rxvt_privileged_utmp(rxvt_t* r, int page, char action)
 {
-    DBG_MSG(1,(stderr, "rxvt_privileged_utmp %d (%c); waiting for: %c (pid: %d)\n", page, action, PVTS(r, page)->next_utmp_action, (int) getpid()));
+    rxvt_msg (DBG_INFO, DBG_MAIN, "rxvt_privileged_utmp %d (%c); waiting for: %c (pid: %d)\n", page, action, PVTS(r, page)->next_utmp_action, (int) getpid());
 
     if (PVTS(r, page)->next_utmp_action != action ||
 	(action != SAVE && action != RESTORE) ||
@@ -704,7 +702,7 @@ rxvt_privileged_utmp(rxvt_t* r, int page, char action)
 void
 rxvt_privileged_ttydev(rxvt_t* r, int page, char action)
 {
-    DBG_MSG(1,(stderr, "rxvt_privileged_ttydev %d (r, %c); waiting for: %c (pid: %d)\n", page, action, PVTS(r, page)->next_tty_action, getpid()));
+    rxvt_msg (DBG_INFO, DBG_MAIN, "rxvt_privileged_ttydev %d (r, %c); waiting for: %c (pid: %d)\n", page, action, PVTS(r, page)->next_tty_action, getpid());
     if (PVTS(r, page)->next_tty_action != action ||
 	(action != SAVE && action != RESTORE) ||
 	IS_NULL(PVTS(r, page)->ttydev) ||
@@ -750,7 +748,7 @@ rxvt_privileged_ttydev(rxvt_t* r, int page, char action)
     rxvt_privileges(IGNORE);
 
 # ifndef RESET_TTY_TO_COMMON_DEFAULTS
-    DBG_MSG(1,(stderr, "%s \"%s\": mode %03o, uid %d, gid %d\n", action == RESTORE ? "Restoring" : (action == SAVE ? "Saving" : "UNKNOWN ERROR for"), PVTS(r, page)->ttydev, PVTS(r, page)->ttyfd_stat.st_mode, PVTS(r, page)->ttyfd_stat.st_uid, PVTS(r, page)->ttyfd_stat.st_gid));
+    rxvt_msg (DBG_INFO, DBG_MAIN, "%s \"%s\": mode %03o, uid %d, gid %d\n", action == RESTORE ? "Restoring" : (action == SAVE ? "Saving" : "UNKNOWN ERROR for"), PVTS(r, page)->ttydev, PVTS(r, page)->ttyfd_stat.st_mode, PVTS(r, page)->ttyfd_stat.st_uid, PVTS(r, page)->ttyfd_stat.st_gid);
 # endif
 }
 #endif
@@ -767,23 +765,33 @@ rxvt_tt_winsize(int fd, unsigned short col, unsigned short row, pid_t pid)
 {
     struct winsize  ws;
 
+#ifdef DEBUG
+    if( pid && fd > STDERR_FILENO )
+    {
+	rxvt_dbgmsg ((DBG_DEBUG, DBG_MAIN, "%s( fd=%d, col=%hu, row=%hu, pid=%d )\n", __func__, fd, col, row, pid));
+    }
+#endif
+
     if (fd < 0)
-    return;
+	return;
+
     ws.ws_col = col;
     ws.ws_row = row;
     ws.ws_xpixel = ws.ws_ypixel = 0;
-#ifndef DEBUG
-    (void)ioctl(fd, TIOCSWINSZ, &ws);
-#else
+
     if (ioctl(fd, TIOCSWINSZ, &ws) < 0)
     {
-	DBG_MSG(1,(stderr, "Failed to send TIOCSWINSZ to fd %d\n", fd));
+#ifdef DEBUG
+	if( pid && fd > STDERR_FILENO )
+	{
+	    rxvt_msg (DBG_WARN, DBG_MAIN, "Failed to send TIOCSWINSZ to fd %d\n", fd);
+	}
+#endif
     }
-# ifdef SIGWINCH
-    else if (pid)	/* force through to the command */
+#ifdef SIGWINCH
+    else if( pid )	/* force through to the command */
 	kill(pid, SIGWINCH);
-# endif
-#endif	/* DEBUG */
+#endif
 }
 
 
@@ -804,7 +812,7 @@ rxvt_init_bfont_xft (rxvt_t* r, XftPattern* xpold)
 # endif
 
 
-    DBG_MSG( 2, ( stderr, "rxvt_init_bfont_xft()" ) );
+    rxvt_dbgmsg ((DBG_VERBOSE, DBG_MAIN, "rxvt_init_bfont_xft()"));
 
     xp = XftPatternDuplicate (xpold);
     if (IS_NULL(xp))
@@ -830,12 +838,12 @@ rxvt_init_bfont_xft (rxvt_t* r, XftPattern* xpold)
 
 	else
 	{
-	    DBG_MSG( 3, ( stderr, "Opened bold font: h=%d(%d), w=%d(%d)",
+	    rxvt_dbgmsg ((DBG_VERBOSE, DBG_MAIN, "Opened bold font: h=%d(%d), w=%d(%d)",
 			r->TermWin.xftbfont->height,
 			r->TermWin.xftfont->height,
 			r->TermWin.xftbfont->max_advance_width,
-			r->TermWin.xftfont->max_advance_width) );
-#ifdef DEBUG_VERBOSE
+			r->TermWin.xftfont->max_advance_width));
+#ifdef DEBUG
 	    FcPatternPrint( xftbpattern );
 #endif
 	    /*
@@ -849,7 +857,7 @@ rxvt_init_bfont_xft (rxvt_t* r, XftPattern* xpold)
 	    {
 		if (r->TermWin.xftbfont->max_advance_width > r->TermWin.fwidth)
 		{
-		    rxvt_print_error( "Bold font too wide. Using overstrike" );
+		    rxvt_msg (DBG_ERROR, DBG_MAIN,  "Bold font too wide. Using overstrike" );
 		    XftFontClose( r->Xdisplay, r->TermWin.xftbfont );
 		    SET_NULL( r->TermWin.xftbfont );
 		}
@@ -886,7 +894,7 @@ rxvt_init_mfont_xft (rxvt_t* r, XftPattern* xp, const char* ofname)
     XftResult	    fr;
     int		    len, olen;	    /* font name length */
     char*	    mfname;	    /* mfont name to open */
-    char*	    omfname = NULL; /* actually opened mfont name */
+    FcChar8*	    omfname = NULL; /* actually opened mfont name */
     int		    width, height;
 # ifdef DEBUG
     FT_Face	    face;
@@ -900,7 +908,7 @@ rxvt_init_mfont_xft (rxvt_t* r, XftPattern* xp, const char* ofname)
     /*
     ** Now try to open freetype mfont
     */
-    DBG_MSG(2,(stderr, "load freetype mfont\n"));
+    rxvt_dbgmsg ((DBG_VERBOSE, DBG_MAIN, "load freetype mfont\n"));
 
     /* font family */
     mfname = (char*) r->h->rs[Rs_xftmfont];
@@ -971,7 +979,7 @@ rxvt_init_mfont_xft (rxvt_t* r, XftPattern* xp, const char* ofname)
 	XftPatternAddBool (r->TermWin.xftmpattern, FC_GLOBAL_ADVANCE, FcTrue);
     }
 
-#  ifdef DEBUG_VERBOSE
+#  ifdef DEBUG
     FcPatternPrint (r->TermWin.xftmpattern);
 #  endif
 
@@ -979,7 +987,7 @@ rxvt_init_mfont_xft (rxvt_t* r, XftPattern* xp, const char* ofname)
      * Print a warning if our matched mfont is different from the user supplied
      * font.
      */
-    XftPatternGetString (r->TermWin.xftmpattern, XFT_FAMILY, 0, &omfname);
+    XftPatternGetString( r->TermWin.xftmpattern, XFT_FAMILY, 0, &omfname );
 
     assert (NOT_NULL(omfname));	/* shouldn't be NULL */
     len = STRLEN(mfname);
@@ -987,14 +995,12 @@ rxvt_init_mfont_xft (rxvt_t* r, XftPattern* xp, const char* ofname)
 
     if( STRCMP( mfname, DEFAULT_XFT_FONT_NAME) 	/* Not the default font */
 	    /* Not opened font */
-	    && (len != olen || STRNCASECMP(omfname, mfname, len))
+	    && (len != olen || STRNCASECMP((char*) omfname, mfname, len))
     )
-	rxvt_print_error ("Cannot open mfont '%s'. Using mfont '%s' instead.",
+	rxvt_msg (DBG_ERROR, DBG_MAIN, "Cannot open mfont '%s'. Using mfont '%s' instead.",
 	    mfname, omfname);
 
-    DBG_MSG(1, (stderr, "create xftmpattern = 0x%x on mfont %d\n",
-	(unsigned int) r->TermWin.xftmpattern, r->h->rs[Rs_xftmsz] ?
-	r->TermWin.xftmsize : r->TermWin.xftfont->height-1));
+    rxvt_dbgmsg ((DBG_DEBUG, DBG_MAIN, "create xftmpattern = 0x%x on mfont %d\n", (unsigned int) r->TermWin.xftmpattern, r->h->rs[Rs_xftmsz] ?  r->TermWin.xftmsize : r->TermWin.xftfont->height-1));
 
     /*
      * Actually open the mfont.
@@ -1015,7 +1021,7 @@ rxvt_init_mfont_xft (rxvt_t* r, XftPattern* xp, const char* ofname)
 	r->TermWin.xftmono = 0;
 
     else if (
-	      STRCASECMP (ofname, omfname)
+	      STRCASECMP (ofname, (char*) omfname)
 	      && (r->TermWin.fwidth != (width >> 1))
 	    )
 	r->TermWin.xftmono = 0;
@@ -1023,7 +1029,7 @@ rxvt_init_mfont_xft (rxvt_t* r, XftPattern* xp, const char* ofname)
     else if (ISSET_OPTION(r, Opt2_xftSlowOutput))
 	r->TermWin.xftmono = 0;
 
-    DBG_MSG(1, (stderr, "xftmono is %d\n", r->TermWin.xftmono));
+    rxvt_dbgmsg ((DBG_VERBOSE, DBG_MAIN, "xftmono is %d\n", r->TermWin.xftmono));
 
     MAX_IT (r->TermWin.fwidth, (width >> 1));
     if( !r->TermWin.xftpfont )
@@ -1095,16 +1101,16 @@ rxvt_init_font_xft (rxvt_t* r)
     XGlyphInfo	    ext1, ext2;
     int		    len, olen;	    /* font name length */
     char*	    fname;	    /* font name to open */
-    char*	    ofname = NULL;  /* actually opened font name */
+    FcChar8*	    ofname = NULL;  /* actually opened font name */
 # ifdef DEBUG
     FT_Face	    face;
 # endif
 
 
-    DBG_MSG(2,(stderr, "rxvt_init_font_xft\n"));
+    rxvt_dbgmsg ((DBG_VERBOSE, DBG_MAIN, "rxvt_init_font_xft\n"));
 
 
-    DBG_MSG(2,(stderr, "load freetype font\n"));
+    rxvt_dbgmsg ((DBG_VERBOSE, DBG_MAIN, "load freetype font\n"));
     xp = XftPatternCreate ();
     if (IS_NULL(xp))
 	return 0;
@@ -1225,7 +1231,7 @@ rxvt_init_font_xft (rxvt_t* r)
 	XftPatternAddBool (r->TermWin.xftpattern, FC_GLOBAL_ADVANCE, FcTrue);
     }
 
-# ifdef DEBUG_VERBOSE
+# ifdef DEBUG
     FcPatternPrint (r->TermWin.xftpattern);
 # endif
 
@@ -1240,13 +1246,12 @@ rxvt_init_font_xft (rxvt_t* r)
     olen = STRLEN(ofname);
     if( STRCMP( fname, DEFAULT_XFT_FONT_NAME) 	/* Not the default font */
 	    /* Not opened font */
-	    && (len != olen || STRNCASECMP(ofname, fname, len))
+	    && (len != olen || STRNCASECMP((char*) ofname, fname, len))
     )
-	rxvt_print_error ("Cannot open font '%s'. Using font '%s' instead.",
+	rxvt_msg (DBG_ERROR, DBG_MAIN, "Cannot open font '%s'. Using font '%s' instead.",
 	    fname, ofname);
 
-    DBG_MSG(1, (stderr, "create xftpattern = 0x%x on font %d\n",
-	(unsigned int) r->TermWin.xftpattern, r->TermWin.xftsize));
+    rxvt_dbgmsg ((DBG_DEBUG, DBG_MAIN, "create xftpattern = 0x%x on font %d\n", (unsigned int) r->TermWin.xftpattern, r->TermWin.xftsize));
 
     /*
      * Actually open the font.
@@ -1322,8 +1327,7 @@ rxvt_init_font_xft (rxvt_t* r)
 
 	XftPatternDestroy( ppat);
 
-	DBG_MSG( 1, ( stderr, "Opened prop fonts %p, %p\n",
-		    r->TermWin.xftpfont, r->TermWin.xftPfont));
+	rxvt_dbgmsg ((DBG_VERBOSE, DBG_MAIN, "Opened prop fonts %p, %p\n", r->TermWin.xftpfont, r->TermWin.xftPfont));
     }
     
     if (IS_NULL(r->TermWin.xftfont))
@@ -1355,7 +1359,7 @@ rxvt_init_font_xft (rxvt_t* r)
 	 * This will however make the font spacing in tab titles incorrect for
 	 * double width fonts.
 	 */
-	DBG_MSG( 1, ( stderr, "Warning: Double width font, with no pfont\n") );
+	rxvt_msg (DBG_WARN, DBG_MAIN, "Warning: Double width font, with no pfont\n");
 	r->TermWin.xftpfont = r->TermWin.xftfont;
 	r->TermWin.xftPfont = r->TermWin.xftfont;
     }
@@ -1384,7 +1388,7 @@ rxvt_init_font_xft (rxvt_t* r)
 #ifdef MULTICHAR_SET
     if( isDoubleWidthFont( r->Xdisplay, r->TermWin.xftfont ) )
     {
-	DBG_MSG( 1, ( stderr, "Got double width font. Halving width\n" ) );
+	rxvt_msg (DBG_INFO, DBG_MAIN, "Got double width font. Halving width\n");
 	r->TermWin.xftmono = 0;
 	r->TermWin.fwidth >>= 1;
     }
@@ -1401,7 +1405,7 @@ rxvt_init_font_xft (rxvt_t* r)
 	else
 	    r->TermWin.xftfnmono = r->TermWin.xftmono = 0;
     }
-    DBG_MSG(1, (stderr, "xftfnmono is %d\n", r->TermWin.xftfnmono));
+    rxvt_dbgmsg ((DBG_VERBOSE, DBG_MAIN, "xftfnmono is %d\n", r->TermWin.xftfnmono));
 
 
 # ifndef NO_BOLDFONT
@@ -1409,7 +1413,7 @@ rxvt_init_font_xft (rxvt_t* r)
 # endif
 
 # ifdef MULTICHAR_SET
-    if (!rxvt_init_mfont_xft (r, xp, ofname))
+    if (!rxvt_init_mfont_xft (r, xp, (char*) ofname))
 	goto Failure;
 # endif	/* MULTICHAR_SET */
 
@@ -1441,11 +1445,11 @@ rxvt_init_font_fixed (rxvt_t* r)
     XFontStruct*    xfont;
 
 
-    DBG_MSG(1,(stderr, " load font (fixed)\n"));
+    rxvt_dbgmsg ((DBG_VERBOSE, DBG_MAIN, " load font (fixed)\n"));
     xfont = XLoadQueryFont (r->Xdisplay, "fixed");
     if (IS_NULL(xfont))
     {
-	rxvt_print_error("fatal error, aborting...");
+	rxvt_msg (DBG_ERROR, DBG_MAIN, "fatal error, aborting...");
 	exit(EXIT_FAILURE);
     }
 
@@ -1478,7 +1482,7 @@ rxvt_init_font_x11 (rxvt_t *r)
     int		    idx = 0;		/* index into rs_font[] */
 
 
-    DBG_MSG(1,(stderr, "rxvt_init_font_x11 \n"));
+    rxvt_dbgmsg ((DBG_VERBOSE, DBG_MAIN, "rxvt_init_font_x11 \n"));
 
 
 #ifdef XFT_SUPPORT
@@ -1495,21 +1499,21 @@ rxvt_init_font_x11 (rxvt_t *r)
     idx = FNUM2IDX(r->h->fnum);
 
     /* OK, now it's time to load the default font */
-    DBG_MSG(1,(stderr, " load font (%s)\n", r->h->rs[Rs_font+idx]));
+    rxvt_dbgmsg ((DBG_VERBOSE, DBG_MAIN, " load font (%s)\n", r->h->rs[Rs_font+idx]));
     xfont = XLoadQueryFont (r->Xdisplay, r->h->rs[Rs_font+idx]);
     if (IS_NULL(xfont))
     {
 	/* failed to load font */
-	rxvt_print_error(msg, r->h->rs[Rs_font+idx]);
+	rxvt_msg (DBG_ERROR, DBG_MAIN, msg, r->h->rs[Rs_font+idx]);
 
 	/* try to load fixed font */
 	r->h->rs[Rs_font+idx] = "fixed";
-	DBG_MSG(1,(stderr, " load font (%s)\n", r->h->rs[Rs_font+idx]));
+	rxvt_dbgmsg ((DBG_VERBOSE, DBG_MAIN, " load font (%s)\n", r->h->rs[Rs_font+idx]));
 	xfont = XLoadQueryFont(r->Xdisplay, r->h->rs[Rs_font+idx]);
 	if (IS_NULL(xfont))
 	{
 	    /* still failed to load font */
-	    rxvt_print_error(msg, r->h->rs[Rs_font+idx]);
+	    rxvt_msg (DBG_ERROR, DBG_MAIN, msg, r->h->rs[Rs_font+idx]);
 
 	    /* cannot load any font, fatal error, abort the program */
 	    goto Abort;
@@ -1555,7 +1559,7 @@ rxvt_init_font_x11 (rxvt_t *r)
 	/* try to load boldFont, fail silently */
 	if (NOT_NULL(r->h->rs[Rs_boldFont+idx]))
 	{
-	    DBG_MSG(1,(stderr, " load bfont (%s)\n", r->h->rs[Rs_boldFont+idx]));
+	    rxvt_dbgmsg ((DBG_VERBOSE, DBG_MAIN, " load bfont (%s)\n", r->h->rs[Rs_boldFont+idx]));
 	    bfont = XLoadQueryFont (r->Xdisplay, r->h->rs[Rs_boldFont+idx]);
 	}
 
@@ -1585,24 +1589,24 @@ rxvt_init_font_x11 (rxvt_t *r)
 
 #ifdef MULTICHAR_SET
     /* load font or substitute */
-    DBG_MSG(1,(stderr, " load mfont (%s)\n", r->h->rs[Rs_mfont+idx]));
+    rxvt_dbgmsg ((DBG_VERBOSE, DBG_MAIN, " load mfont (%s)\n", r->h->rs[Rs_mfont+idx]));
     xfont = XLoadQueryFont(r->Xdisplay, r->h->rs[Rs_mfont+idx]);
     if (IS_NULL(xfont))
     {
 	char*	ptr;
 
 	/* failed to load font */
-	rxvt_print_error(msg, r->h->rs[Rs_mfont+idx]);
+	rxvt_msg (DBG_ERROR, DBG_MAIN, msg, r->h->rs[Rs_mfont+idx]);
 
 	ptr = rxvt_fallback_mfont_x11 (r);
-	DBG_MSG(1,(stderr, " load mfont (%s)\n", ptr));
+	rxvt_dbgmsg ((DBG_VERBOSE, DBG_MAIN, " load mfont (%s)\n", ptr));
 	xfont = XLoadQueryFont(r->Xdisplay, ptr);
 	if (NOT_NULL(xfont))
 	    r->h->rs[Rs_mfont+idx] = ptr;
 	else
 	{
 	    /* still failed to load font */
-	    rxvt_print_error(msg, ptr);
+	    rxvt_msg (DBG_ERROR, DBG_MAIN, msg, ptr);
 	    /* cannot load any mfont, fatal error, abort the program */
 	    goto Abort;
 	}
@@ -1617,7 +1621,7 @@ rxvt_init_font_x11 (rxvt_t *r)
     return ;
 
 Abort:
-    rxvt_print_error("fatal error, aborting...");
+    rxvt_msg (DBG_ERROR, DBG_MAIN, "fatal error, aborting...");
     exit(EXIT_FAILURE);
 }
 
@@ -1723,7 +1727,7 @@ rxvt_change_font_xft (rxvt_t* r, const char* fontname)
 
 
     assert (fontname);
-    DBG_MSG(2,(stderr, "rxvt_change_font_xft (%s)\n", fontname));
+    rxvt_dbgmsg ((DBG_VERBOSE, DBG_MAIN, "rxvt_change_font_xft (%s)\n", fontname));
 
     /* we only accept FONT_CMD now for XFT font ;-) */
     if (FONT_CMD != fontname[0])
@@ -1825,8 +1829,7 @@ rxvt_change_font_xft (rxvt_t* r, const char* fontname)
      */
 
 #if 0
-    DBG_MSG(1, (stderr, "Destroy %p, (xftpattern %p, xftmpattern %p)\n",
-		xp, r->TermWin.xftpattern, r->TermWin.xftmpattern));
+    rxvt_dbgmsg ((DBG_DEBUG, DBG_MAIN, "Destroy %p, (xftpattern %p, xftmpattern %p)\n", xp, r->TermWin.xftpattern, r->TermWin.xftmpattern)));
     /* FcPatternPrint (xp);
     FcPatternPrint (r->TermWin.xftpattern); */
     XftPatternDestroy (xp);
@@ -1864,7 +1867,7 @@ rxvt_change_font_x11 (rxvt_t* r, const char *fontname)
 
 
     assert (fontname);
-    DBG_MSG(1,(stderr, "rxvt_change_font_x11 (%s)\n", fontname));
+    rxvt_dbgmsg ((DBG_VERBOSE, DBG_MAIN, "rxvt_change_font_x11 (%s)\n", fontname));
 
 
     switch (fontname[0])
@@ -1950,21 +1953,21 @@ rxvt_change_font_x11 (rxvt_t* r, const char *fontname)
     /*
     ** OK, now it's time to load font or substitute
     */
-    DBG_MSG(1,(stderr, " load font (%s)\n", r->h->rs[Rs_font+idx]));
+    rxvt_dbgmsg ((DBG_VERBOSE, DBG_MAIN, " load font (%s)\n", r->h->rs[Rs_font+idx]));
     xfont = XLoadQueryFont (r->Xdisplay, r->h->rs[Rs_font+idx]);
     if (!xfont)
     {
 	/* failed to load font */
-	rxvt_print_error(msg, r->h->rs[Rs_font+idx]);
+	rxvt_msg (DBG_ERROR, DBG_MAIN, msg, r->h->rs[Rs_font+idx]);
 
 	/* try to load fixed font */
 	r->h->rs[Rs_font+idx] = "fixed";
-	DBG_MSG(1,(stderr, " load font (%s)\n", r->h->rs[Rs_font+idx]));
+	rxvt_dbgmsg ((DBG_VERBOSE, DBG_MAIN, " load font (%s)\n", r->h->rs[Rs_font+idx]));
 	xfont = XLoadQueryFont(r->Xdisplay, r->h->rs[Rs_font+idx]);
 	if (!xfont)
 	{
 	    /* still failed to load font */
-	    rxvt_print_error(msg, r->h->rs[Rs_font+idx]);
+	    rxvt_msg (DBG_ERROR, DBG_MAIN, msg, r->h->rs[Rs_font+idx]);
 	    return 0;
 	}
     }
@@ -2026,7 +2029,7 @@ rxvt_change_font_x11 (rxvt_t* r, const char *fontname)
 	if (IS_NULL(r->TermWin.bfont) &&
 	    NOT_NULL(r->h->rs[Rs_boldFont+idx]))
 	{
-	    DBG_MSG(1,(stderr, " load bfont (%s)\n", r->h->rs[Rs_boldFont+idx]));
+	    rxvt_dbgmsg ((DBG_VERBOSE, DBG_MAIN, " load bfont (%s)\n", r->h->rs[Rs_boldFont+idx]));
 	    bfont = XLoadQueryFont (r->Xdisplay, r->h->rs[Rs_boldFont+idx]);
 	}
 
@@ -2067,32 +2070,32 @@ rxvt_change_font_x11 (rxvt_t* r, const char *fontname)
 #endif
 	IS_WIN(r->menuBar.win))
     {
-	DBG_MSG( 3, ( stderr, "Resized menubar font\n"));
+	rxvt_dbgmsg ((DBG_VERBOSE, DBG_MAIN, "Resized menubar font\n"));
 	XSetFont( r->Xdisplay, r->menuBar.gc, r->TermWin.font->fid);
     }
 #endif
 
 #ifdef MULTICHAR_SET
     /* load font or substitute */
-    DBG_MSG(1,(stderr, " load mfont (%s)\n", r->h->rs[Rs_mfont+idx]));
+    rxvt_dbgmsg ((DBG_VERBOSE, DBG_MAIN, " load mfont (%s)\n", r->h->rs[Rs_mfont+idx]));
     xfont = XLoadQueryFont(r->Xdisplay, r->h->rs[Rs_mfont+idx]);
     if (!xfont)
     {
 	char*	ptr;
 
 	/* failed to load font */
-	rxvt_print_error(msg, r->h->rs[Rs_mfont+idx]);
+	rxvt_msg (DBG_ERROR, DBG_MAIN, msg, r->h->rs[Rs_mfont+idx]);
 
 	/* try to load default font */
 	ptr = rxvt_fallback_mfont_x11 (r);
-	DBG_MSG(1,(stderr, " load mfont (%s)\n", ptr));
+	rxvt_dbgmsg ((DBG_VERBOSE, DBG_MAIN, " load mfont (%s)\n", ptr));
 	xfont = XLoadQueryFont(r->Xdisplay, ptr);
 	if (xfont)
 	    r->h->rs[Rs_mfont+idx] = ptr;
 	else
 	{
 	    /* still failed to load font */
-	    rxvt_print_error(msg, ptr);
+	    rxvt_msg (DBG_ERROR, DBG_MAIN, msg, ptr);
 	    return 0;
 	}
     }
@@ -2251,8 +2254,7 @@ rxvt_set_window_color(rxvt_t* r, int page, int idx, const char *color)
     register int    i;
 
 
-    DBG_MSG( 2, ( stderr, "%s( r, %d, %d, %s), ATAB=%d\n",
-		__func__, page, idx, color, ATAB(r) ) );
+    rxvt_dbgmsg ((DBG_DEBUG, DBG_MAIN, "%s( r, %d, %d, %s), ATAB=%d\n", __func__, page, idx, color, ATAB(r)));
 
     if (IS_NULL(color) || (char) 0 == *color)
 	return;
@@ -2418,8 +2420,7 @@ rxvt_recolour_cursor(rxvt_t *r)
     XQueryColors(r->Xdisplay, XCMAP, xcol, 2);
     XRecolorCursor(r->Xdisplay, r->term_pointer, &(xcol[0]), &(xcol[1]));
 
-    DBG_MSG( 3, ( stderr, "%s(r): fg=%06lx, bg=%06lx\n",
-		__func__, xcol[0].pixel, xcol[1].pixel ) );
+    rxvt_dbgmsg ((DBG_DEBUG, DBG_MAIN, "%s(r): fg=%06lx, bg=%06lx\n", __func__, xcol[0].pixel, xcol[1].pixel));
 }
 
 
@@ -2526,7 +2527,7 @@ rxvt_parse_alloc_color(rxvt_t* r, XColor *screen_in_out, const char *colour)
     int		    res = 0;
 
     if (!XParseColor(r->Xdisplay, XCMAP, colour, screen_in_out))
-	rxvt_print_error("can't determine colour: %s", colour);
+	rxvt_msg (DBG_ERROR, DBG_MAIN, "can't determine colour: %s", colour);
     else
 	res = rxvt_alloc_color(r, screen_in_out, colour);
     return res;
@@ -2586,7 +2587,7 @@ rxvt_alloc_color( rxvt_t* r, XColor *screen_in_out, const char *colour )
     }
 
     if (res == 0)
-	rxvt_print_error("can't allocate color: %s", colour);
+	rxvt_msg (DBG_ERROR, DBG_MAIN, "can't allocate color: %s", colour);
 
     return res;
 #endif
@@ -2641,7 +2642,7 @@ rxvt_IM_is_running(rxvt_t *r)
     char	    server[IMBUFSIZ];
 
     /* get current locale modifier */
-    DBG_MSG(2, (stderr, "rxvt_IM_is_running ()\n"));
+    rxvt_dbgmsg ((DBG_VERBOSE, DBG_MAIN, "rxvt_IM_is_running ()\n"));
     if (NOT_NULL(p = XSetLocaleModifiers(NULL)))
     {
 	STRCPY(server, "@server=");
@@ -2696,7 +2697,7 @@ rxvt_IM_set_fontset (rxvt_t* r, int idx)
 
     if (idx < 0 || idx >= MAX_NFONTS)
 	return;
-    DBG_MSG(1,(stderr, "rxvt_setTermFontSet()\n"));
+    rxvt_dbgmsg ((DBG_VERBOSE, DBG_MAIN, "rxvt_setTermFontSet()\n"));
     prev_fontset = r->TermWin.fontset;
     SET_NULL(r->TermWin.fontset);
 
@@ -2822,7 +2823,7 @@ rxvt_IM_init_callback (Display *unused __attribute__((unused)), XPointer client_
     rxvt_t	    *r = rxvt_get_r();
     char	    buf[IMBUFSIZ];
 
-    DBG_MSG(1,(stderr, "rxvt_IM_init_callback()\n"));
+    rxvt_dbgmsg ((DBG_VERBOSE, DBG_MAIN, "rxvt_IM_init_callback()\n"));
     if (r->h->Input_Context)
 	return;
 
@@ -2891,11 +2892,11 @@ rxvt_IM_get_IC(rxvt_t *r)
     struct rxvt_hidden *h = r->h;
     long	    im_event_mask = 0;
 
-    DBG_MSG(1,(stderr, "rxvt_IM_get_IC()\n"));
+    rxvt_dbgmsg ((DBG_VERBOSE, DBG_MAIN, "rxvt_IM_get_IC()\n"));
     xim = XOpenIM(r->Xdisplay, NULL, NULL, NULL);
     if (IS_NULL(xim))
     {
-	DBG_MSG(1,(stderr, "Unalbe to open IM\n"));
+	rxvt_msg (DBG_WARN, DBG_MAIN, "Unalbe to open IM\n");
 	return False;
     }
 
@@ -2997,7 +2998,7 @@ rxvt_IM_get_IC(rxvt_t *r)
 	XFree(status_attr);
     if (IS_NULL(h->Input_Context))
     {
-	rxvt_print_error("failed to create input context");
+	rxvt_msg (DBG_ERROR, DBG_MAIN, "failed to create input context");
 	XCloseIM(xim);
 	return False;
     }
@@ -3017,7 +3018,7 @@ rxvt_IM_get_IC(rxvt_t *r)
     XSelectInput(r->Xdisplay, r->TermWin.parent, im_event_mask);
 
 
-    DBG_MSG(1,(stderr, "rxvt_IM_get_IC() - successful connection\n"));
+    rxvt_msg (DBG_INFO, DBG_MAIN, "rxvt_IM_get_IC() - successful connection\n");
     return True;
 }
 
