@@ -27,17 +27,6 @@
 
 #define NEWARGLIM 500
 
-#ifdef DEBUG_VERBOSE
-#define DEBUG_LEVEL 1
-#else 
-#define DEBUG_LEVEL 0
-#endif
-
-#if DEBUG_LEVEL
-#define DBG_MSG(d,x) if(d <= DEBUG_LEVEL) fprintf x
-#else
-#define DBG_MSG(d,x)
-#endif
 
 #ifdef HAVE_MENUBAR
 
@@ -75,7 +64,7 @@ void          rxvt_draw_arrows            (rxvt_t*, int, int);
 void          rxvt_menubar_select         (rxvt_t*, XButtonEvent*);
 void	      rxvt_menubar_draw_labels	  (rxvt_t*);
 void	      resizeSubMenus		  (rxvt_t*, menu_t*);
-#ifdef DEBUG_MENU_LAYOUT
+#ifdef DEBUG
 void          rxvt_print_menu_ancestors   (menu_t*);
 void          rxvt_print_menu_descendants (menu_t*);
 #endif
@@ -139,12 +128,12 @@ rxvt_menuitem_find(const menu_t *menu, const unsigned char *name)
     assert(NOT_NULL(menu));
 #endif
 
-    DBG_MSG( 2, (stderr, "rxvt_menuitem_find(%s) ... ", name));
+    rxvt_dbgmsg ((DBG_DEBUG, DBG_MENUBAR, "rxvt_menuitem_find(%s) ... ", name));
 
     /* find the last item in the menu, this is good for separators */
     for (item = menu->tail; NOT_NULL(item); item = item->prev)
     {
-	DBG_MSG(4, ( stderr, "%d", item->entry.itemType));
+	rxvt_dbgmsg ((DBG_DEBUG, DBG_MENUBAR, "%d", item->entry.itemType));
 
 	if (item->entry.itemType == MenuSubMenu)
 	{
@@ -155,10 +144,10 @@ rxvt_menuitem_find(const menu_t *menu, const unsigned char *name)
 	       || !STRCMP(name, item->name))
 	    break;
 
-	DBG_MSG( 4, ( stderr, ","));
+	rxvt_dbgmsg ((DBG_DEBUG, DBG_MENUBAR, ","));
     }
 
-    DBG_MSG( 2, (stderr, "Returning %p\n", item));
+    rxvt_dbgmsg ((DBG_DEBUG, DBG_MENUBAR, "Returning %p\n", item));
     return item;
 }
 
@@ -175,7 +164,7 @@ rxvt_menuitem_free(rxvt_t *r, menu_t *menu, menuitem_t *item)
 #ifdef DEBUG
     assert(NOT_NULL(menu));
 #endif
-    DBG_MSG( 2, (stderr, "rxvt_menuitem_free()\n"));
+    rxvt_dbgmsg ((DBG_DEBUG, DBG_MENUBAR, "rxvt_menuitem_free()\n"));
 
     prev = item->prev;
     next = item->next;
@@ -204,134 +193,6 @@ rxvt_menuitem_free(rxvt_t *r, menu_t *menu, menuitem_t *item)
     rxvt_free(item);
 }
 
-#if 0 /* {{{ Obsolete: macro action dispatch / set are much better. */
-/*
- * sort command vs. terminal actions and
- * remove the first character of STR if it's '\0'
- *
- * return value < 0 means caller can safely free str. >=0 0 means was assigned
- * to action, and should only be freed when action is destroyed.
- */
-/* INTPROTO */
-int
-rxvt_action_type(action_t *action, unsigned char *str)
-{
-    unsigned short  len;
-
-    DBG_MSG( 2, (stderr, "rxvt_action_type(action, %s)\n", str));
-
-    switch( *str)
-    {
-	    /*
-	     * Old behaviour: Use ^@^@ for leading null strings, and ^@ for
-	     * terminal strings.
-	     */
-#if 0
-	case '>':   /* Write string to child process */
-#if defined (DEBUG_MENU) || defined (DEBUG_MENUARROWS)
-	    len = STRLEN(str+1);
-	    fprintf(stderr, "(len %d) = %s\n", len, str);
-#else
-	    len = rxvt_str_escaped( ((char*) str) + 1); /* Skip leading '>' */
-#endif
-
-	    if (!len) return -1;
-
-	    /* sort command vs. terminal actions */
-	    action->type = MenuItem;
-	    if (str[1] == '\0')
-	    {
-		/* the functional equivalent: memmove (str, str+2, len); */
-		unsigned char  *dst = (str);
-		unsigned char  *src = (str + 2);
-		unsigned char  *end = (str + len);
-
-		while (src <= end)
-		    *dst++ = *src++;
-
-		len--;		/* decrement length */
-		if (str[0] != '\0')
-		    action->type = MenuTerminalAction;
-	    }
-	    else
-		/* Skip leading '>' */
-		memmove( str, str + 1, len);
-
-	    action->str = str;
-	    action->len = len;
-	    break;
-#endif
-
-	case '>':   /* Write string to child process */
-	case ']':   /* Write string to rxvt */
-	    action->type = ((*str == '>') ? MenuAction : MenuTerminalAction);
-
-	    len = rxvt_str_escaped( ((char*) str) + 1); /* Skip leading ] or > */
-	    memmove( str, str + 1, len);
-
-	    action->str = str;
-	    action->len = len;
-	    break;
-
-	case '\0':  /* Empty action */
-	    return -1;
-
-	default:
-#if 0
-	    /*
-	     * Looks like there is no need to implement a new set of functions
-	     * to be called from the menu. Escape sequences work wonderfully.
-	     */
-	    for( i=0; i < NUM_HKFUNCS; i++)
-	    {
-		if( !STRCMP( hk_handlers[i].res+7, str))
-		{
-		    fprintf( stderr, "Matched handler %d, %s\n", i, str);
-
-		    action->type = MenuCallFuncAction;
-		    action->handler = hk_handlers[i].handler;
-
-		    return -1;
-		}
-	    }
-	    fprintf( stderr, "Unknown function %s\n", str);
-#endif
-	    rxvt_print_error( "Badly formed menu action %s", str);
-	    return -1;
-    }
-
-    return 0;
-}
-
-/* INTPROTO */
-int
-rxvt_action_dispatch(rxvt_t *r, action_t *action)
-{
-    DBG_MSG( 2, (stderr, "rxvt_action_dispatch()\n"));
-    switch (action->type)
-    {
-	case MenuTerminalAction:
-	    rxvt_cmd_write(r, ATAB(r), action->str, action->len);
-	    break;
-
-	case MenuAction:
-	    rxvt_tt_write(r, ATAB(r), action->str, action->len);
-	    break;
-
-#if 0	/* Better to implement with escape sequences */
-	case MenuCallFuncAction:
-	    fprintf( stderr, "Calls not implemnted yet\n");
-	    break;
-#endif
-
-	default:
-	    return -1;
-	    break;
-    }
-
-    return 0;
-}
-#endif /* }}} */
 
 /* return the arrow index corresponding to NAME */
 /* INTPROTO */
@@ -340,7 +201,7 @@ rxvt_menuarrow_find(char name)
 {
     int		 i;
 
-    DBG_MSG( 2, (stderr, "rxvt_menuarrow_find()\n"));
+    rxvt_dbgmsg ((DBG_DEBUG, DBG_MENUBAR, "rxvt_menuarrow_find()\n"));
 
     for (i = 0; i < NARROWS; i++)
     if (name == Arrows[i].name)
@@ -354,7 +215,7 @@ void
 rxvt_menuarrow_free(rxvt_t *r, unsigned char name)
 {
     int		 i;
-    DBG_MSG( 2, (stderr, "rxvt_menuarrow_free()\n"));
+    rxvt_dbgmsg ((DBG_DEBUG, DBG_MENUBAR, "rxvt_menuarrow_free()\n"));
 
     if (name)
     {
@@ -404,14 +265,12 @@ rxvt_menuarrow_add(rxvt_t *r, unsigned char *string)
 	    *cur,
 	    parse[NARROWS];
 
-    DBG_MSG( 2, (stderr, "rxvt_menuarrow_add()\n"));
+    rxvt_dbgmsg ((DBG_DEBUG, DBG_MENUBAR, "rxvt_menuarrow_add()\n"));
     MEMSET(parse, 0, sizeof(parse));
 
-/* fprintf(stderr, "add arrows = `%s'\n", string); */
     for (p = string; NOT_NULL(p) && *p; string = p)
     {
     p = (string + 3);
-    /* fprintf(stderr, "parsing at %s\n", string); */
     switch (string[1])
     {
 	case 'b':
@@ -466,20 +325,16 @@ rxvt_menuarrow_add(rxvt_t *r, unsigned char *string)
     cur->len = (p - string);
     }
 
-#ifdef DEBUG_MENUARROWS
+#ifdef DEBUG
     cur = &beg;
-    fprintf(stderr, "<b>(len %d) = %.*s\n",
-	cur->len, cur->len, (cur->str ? cur->str : ""));
+    rxvt_dbgmsg ((DBG_DEBUG, DBG_MENUBAR, "<b>(len %d) = %.*s\n", cur->len, (cur->str ? (char*) cur->str : "")));
     for (i = 0; i < NARROWS; i++)
     {
 	cur = &(parse[i]);
-	fprintf(stderr, "<%c>(len %d) = %.*s\n",
-	    Arrows[i].name,
-	    cur->len, cur->len, (cur->str ? cur->str : ""));
+	rxvt_dbgmsg ((DBG_DEBUG, DBG_MENUBAR, "<%c>(len %d) = %.*s\n", Arrows[i].name, cur->len, (cur->str ? (char*) cur->str : "")));
     }
     cur = &end;
-    fprintf(stderr, "<e>(len %d) = %.*s\n",
-	cur->len, cur->len, (cur->str ? cur->str : ""));
+    rxvt_dbgmsg ((DBG_DEBUG, DBG_MENUBAR, "<e>(len %d) = %.*s\n", cur->len, (cur->str ? (char*) cur->str : "")));
 #endif
 
     xtra_len = (beg.len + end.len);
@@ -517,8 +372,8 @@ rxvt_menuarrow_add(rxvt_t *r, unsigned char *string)
 	}
 	str[len] = '\0';
 
-#ifdef DEBUG_MENUARROWS
-	fprintf(stderr, "<%c>(len %d) = %s\n", Arrows[i].name, len, str);
+#ifdef DEBUG
+	rxvt_dbgmsg ((DBG_DEBUG, DBG_MENUBAR, "<%c>(len %d) = %s\n", Arrows[i].name, len, str));
 #endif
 	rxvt_set_action( &(r->h->MenuBar.arrows[i]), str);
     }
@@ -532,7 +387,7 @@ rxvt_menuitem_add(rxvt_t *r, menu_t *menu,
 {
     menuitem_t	 *item;
     unsigned int    len;
-    DBG_MSG( 2, (stderr, "rxvt_menuitem_add()\n"));
+    rxvt_dbgmsg ((DBG_DEBUG, DBG_MENUBAR, "rxvt_menuitem_add()\n"));
 
 #ifdef DEBUG
     assert(NOT_NULL(name));
@@ -667,7 +522,7 @@ rxvt_menu_find_base(rxvt_t *r, menu_t **menu, unsigned char *path)
 {
     menu_t	 *m = NULL;
     menuitem_t	 *item;
-    DBG_MSG( 2, (stderr, "rxvt_menu_find_base()\n"));
+    rxvt_dbgmsg ((DBG_DEBUG, DBG_MENUBAR, "rxvt_menu_find_base()\n"));
 
 #ifdef DEBUG
     assert(NOT_NULL(menu));
@@ -769,7 +624,7 @@ rxvt_menu_delete(rxvt_t *r, menu_t *menu)
     menu_t	 *parent = NULL, *prev, *next;
     menuitem_t	 *item;
 
-    DBG_MSG( 2, (stderr, "rxvt_menu_delete()\n"));
+    rxvt_dbgmsg ((DBG_DEBUG, DBG_MENUBAR, "rxvt_menu_delete()\n"));
     assert( menu != r->h->ActiveMenu ); /* Shouldn't delete current menu */
 
     /* delete the entire menu */
@@ -842,8 +697,7 @@ rxvt_menu_add(rxvt_t *r, menu_t *parent, unsigned char *path)
 {
     menu_t	 *menu;
 
-    DBG_MSG( 2, (stderr, "rxvt_menu_add(): Adding menu %s to parent %s\n",
-	    path, NOT_NULL(parent) ? parent->name : "(nil)"));
+    rxvt_dbgmsg ((DBG_DEBUG, DBG_MENUBAR, "rxvt_menu_add(): Adding menu %s to parent %s\n", path, NOT_NULL(parent) ? (char *)parent->name : "(nil)"));
 
     if (NOT_NULL(STRCHR(path, '/')))
     {
@@ -959,7 +813,7 @@ rxvt_drawbox_menubar(rxvt_t *r, int x, int len, int state)
 {
     unsigned long   top, bot;
 
-    DBG_MSG( 2, (stderr, "rxvt_drawbox_menubar()\n"));
+    rxvt_dbgmsg ((DBG_DEBUG, DBG_MENUBAR, "rxvt_drawbox_menubar()\n"));
 
     /* XXXMENU */
     len += HSPACE_PIXEL + SHADOW;
@@ -999,7 +853,7 @@ rxvt_menubar_draw_triangle(rxvt_t *r, int x, int y, int state)
     unsigned long   top, bot;
     int		    w;
 
-    DBG_MSG( 2, (stderr, "rxvt_menubar_draw_triangle()\n"));
+    rxvt_dbgmsg ((DBG_DEBUG, DBG_MENUBAR, "rxvt_menubar_draw_triangle()\n"));
 
 #ifdef MENU_SHADOW_IN
     state = -state;
@@ -1034,7 +888,7 @@ rxvt_drawbox_menuitem(rxvt_t *r, int y, int state)
 {
     unsigned long   top, bot;
 
-    DBG_MSG( 2, (stderr, "rxvt_drawbox_menuitem()\n"));
+    rxvt_dbgmsg ((DBG_DEBUG, DBG_MENUBAR, "rxvt_drawbox_menuitem()\n"));
 
 #ifdef MENU_SHADOW_IN
     state = -state;
@@ -1062,40 +916,38 @@ rxvt_drawbox_menuitem(rxvt_t *r, int y, int state)
     XFlush(r->Xdisplay);
 }
 
-#ifdef DEBUG_MENU_LAYOUT
+#ifdef DEBUG
 /* INTPROTO */
 void
 rxvt_print_menu_ancestors(menu_t *menu)
 {
-    DBG_MSG( 2, (stderr, "rxvt_print_menu_ancestors()\n"));
+    rxvt_dbgmsg ((DBG_DEBUG, DBG_MENUBAR, "rxvt_print_menu_ancestors()\n"));
 
     if (IS_NULL(menu))
     {
-	fprintf(stderr, "Top Level menu\n");
+	rxvt_dbgmsg ((DBG_DEBUG, DBG_MENUBAR, "Top Level menu\n"));
 	return;
     }
-    fprintf(stderr, "menu %s ", menu->name);
+    rxvt_dbgmsg ((DBG_DEBUG, DBG_MENUBAR, "menu %s ", menu->name));
     if (NOT_NULL(menu->parent))
     {
 	menuitem_t   *item;
 
 	for (item = menu->parent->head; NOT_NULL(item); item = item->next)
 	{
-	    if (
-		    item->entry.type == MenuSubMenu
-		    && item->entry.submenu.menu == menu
-	       )
+	    if (item->entry.itemType == MenuSubMenu &&
+		item->entry.submenu.menu == menu)
 	    {
 		break;
 	    }
 	}
 	if (IS_NULL(item))
 	{
-	    fprintf(stderr, "is an orphan!\n");
+	    rxvt_dbgmsg ((DBG_DEBUG, DBG_MENUBAR, "is an orphan!\n"));
 	    return;
 	}
     }
-    fprintf(stderr, "\n");
+    rxvt_dbgmsg ((DBG_DEBUG, DBG_MENUBAR, "\n"));
     rxvt_print_menu_ancestors(menu->parent);
 }
 
@@ -1103,7 +955,7 @@ rxvt_print_menu_ancestors(menu_t *menu)
 void
 rxvt_print_menu_descendants(menu_t *menu)
 {
-    DBG_MSG( 2, (stderr, "rxvt_print_menu_descendants()\n"));
+    rxvt_dbgmsg ((DBG_DEBUG, DBG_MENUBAR, "rxvt_print_menu_descendants()\n"));
 
     menuitem_t	 *item;
     menu_t	 *parent;
@@ -1118,33 +970,33 @@ rxvt_print_menu_descendants(menu_t *menu)
     while (NOT_NULL(parent));
 
     for (i = 0; i < level; i++)
-    fprintf(stderr, ">");
-    fprintf(stderr, "%s\n", menu->name);
+    rxvt_dbgmsg ((DBG_DEBUG, DBG_MENUBAR, ">"));
+    rxvt_dbgmsg ((DBG_DEBUG, DBG_MENUBAR, "%s\n", menu->name));
 
     for (item = menu->head; NOT_NULL(item); item = item->next)
     {
-	if (item->entry.type == MenuSubMenu)
+	if (item->entry.itemType == MenuSubMenu)
 	{
 	    if (IS_NULL(item->entry.submenu.menu))
-		fprintf(stderr, "> %s == NULL\n", item->name);
+		rxvt_dbgmsg ((DBG_DEBUG, DBG_MENUBAR, "> %s == NULL\n", item->name));
 	    else
-	    rxvt_print_menu_descendants(item->entry.submenu.menu);
+		rxvt_print_menu_descendants(item->entry.submenu.menu);
 	}
 	else
 	{
 	    for (i = 0; i < level; i++)
-		fprintf(stderr, "+");
-	    if (item->entry.type == MenuLabel)
-		fprintf(stderr, "label: ");
-	    fprintf(stderr, "%s\n", item->name);
+		rxvt_dbgmsg ((DBG_DEBUG, DBG_MENUBAR, "+"));
+	    if (item->entry.itemType == MenuLabel)
+		rxvt_dbgmsg ((DBG_DEBUG, DBG_MENUBAR, "label: "));
+	    rxvt_dbgmsg ((DBG_DEBUG, DBG_MENUBAR, "%s\n", item->name));
 	}
     }
 
     for (i = 0; i < level; i++)
-    fprintf(stderr, "<");
-    fprintf(stderr, "\n");
+    rxvt_dbgmsg ((DBG_DEBUG, DBG_MENUBAR, "<"));
+    rxvt_dbgmsg ((DBG_DEBUG, DBG_MENUBAR, "\n"));
 }
-#endif
+#endif	/* DEBUG */
 
 /*
  * Build a menu with all the titles of current tabs.
@@ -1158,14 +1010,14 @@ rxvt_build_tablist(rxvt_t *r, menu_t *menu)
 		    title[MAX_DISPLAY_TAB_TXT + 4];
 
     assert( menu);
-    DBG_MSG( 2, (stderr, "rxvt_build_tablist()\n"));
+    rxvt_dbgmsg ((DBG_DEBUG, DBG_MENUBAR, "rxvt_build_tablist()\n"));
 
     /*
      * Empty menus should get renamed to "Switch to tab".
      */
     if( !menu->head && STRCMP( menu->name, "Switch to tab"))
     {
-	DBG_MSG( 1, (stderr, "Renaming menu to Switch to tab\n"));
+	rxvt_dbgmsg ((DBG_DEBUG, DBG_MENUBAR, "Renaming menu to Switch to tab\n"));
 
 	menu->name = rxvt_realloc( menu->name, 14);
 	STRCPY( menu->name, "Switch to tab");
@@ -1181,8 +1033,7 @@ rxvt_build_tablist(rxvt_t *r, menu_t *menu)
 
 	rxvt_menuitem_add( r, menu, title, NULL, action);
 
-	DBG_MSG( 1, (stderr, "Added menuentry for %s\n",
-		    PVTS(r, i)->tab_title));
+	rxvt_dbgmsg ((DBG_DEBUG, DBG_MENUBAR, "Added menuentry for %s\n", PVTS(r, i)->tab_title));
     }
 }
 
@@ -1198,7 +1049,7 @@ rxvt_menu_show(rxvt_t *r)
     unsigned short amenu_width;
 
 
-    DBG_MSG( 2, (stderr, "rxvt_menu_show()\n"));
+    rxvt_dbgmsg ((DBG_DEBUG, DBG_MENUBAR, "rxvt_menu_show()\n"));
 
     if (IS_NULL(ActiveMenu))
 	return;
@@ -1507,7 +1358,7 @@ rxvt_menu_display(rxvt_t *r, void (*update)(rxvt_t *))
 {
     menu_t	 *ActiveMenu = r->h->ActiveMenu;
 
-    DBG_MSG( 2, (stderr, "rxvt_menu_display()\n"));
+    rxvt_dbgmsg ((DBG_DEBUG, DBG_MENUBAR, "rxvt_menu_display()\n"));
 
     if (IS_NULL(ActiveMenu))
 	return;
@@ -1538,7 +1389,7 @@ rxvt_menu_display(rxvt_t *r, void (*update)(rxvt_t *))
 void
 rxvt_menu_hide_all(rxvt_t *r)
 {
-    DBG_MSG( 2, (stderr, "rxvt_menu_hide_all()\n"));
+    rxvt_dbgmsg ((DBG_DEBUG, DBG_MENUBAR, "rxvt_menu_hide_all()\n"));
 
     rxvt_menu_display(r, rxvt_menu_hide_all);
     /* r->h->showingMenu &= ~PULLDOWN_MENU; */
@@ -1551,7 +1402,7 @@ rxvt_menu_hide_all(rxvt_t *r)
 void
 rxvt_menu_hide(rxvt_t *r)
 {
-    DBG_MSG( 2, (stderr, "rxvt_menu_hide()\n"));
+    rxvt_dbgmsg ((DBG_DEBUG, DBG_MENUBAR, "rxvt_menu_hide()\n"));
 
     /*
      * rxvt_menu_display just destroys the current menu window, and reshow's the
@@ -1564,7 +1415,7 @@ rxvt_menu_hide(rxvt_t *r)
 void
 rxvt_menu_clear(rxvt_t *r, menu_t *menu)
 {
-    DBG_MSG( 2, (stderr, "rxvt_menu_clear()\n"));
+    rxvt_dbgmsg ((DBG_DEBUG, DBG_MENUBAR, "rxvt_menu_clear()\n"));
 
     if (NOT_NULL(menu))
     {
@@ -1587,7 +1438,7 @@ void
 rxvt_menubar_clear(rxvt_t *r)
 {
     menu_t	 *menu = r->h->MenuBar.tail;
-    DBG_MSG( 2, (stderr, "rxvt_menubar_clear()\n"));
+    rxvt_dbgmsg ((DBG_DEBUG, DBG_MENUBAR, "rxvt_menubar_clear()\n"));
 
     while (NOT_NULL(menu))
     {
@@ -1616,7 +1467,7 @@ rxvt_draw_arrows(rxvt_t *r, int name, int state)
 
     int		 i;
 
-    DBG_MSG( 2, (stderr, "rxvt_draw_arrows()\n"));
+    rxvt_dbgmsg ((DBG_DEBUG, DBG_MENUBAR, "rxvt_draw_arrows()\n"));
 
 #ifdef MENU_SHADOW_IN
     state = -state;
@@ -1670,8 +1521,7 @@ rxvt_menu_select(rxvt_t *r, XButtonEvent *ev)
     int		    unused_root_x, unused_root_y;
     unsigned int    unused_mask;
 
-    DBG_MSG( 2, (stderr, "rxvt_menu_select() (%d)\n",
-		ev->type));
+    rxvt_dbgmsg ((DBG_DEBUG, DBG_MENUBAR, "rxvt_menu_select() (%d)\n", ev->type));
 
     if (IS_NULL(ActiveMenu)) return 0;
 
@@ -1744,7 +1594,7 @@ rxvt_menu_select(rxvt_t *r, XButtonEvent *ev)
 	    switch (ev->type)
 	    {
 		case ButtonRelease:
-		    DBG_MSG( 2, ( stderr, "Got button release\n"));
+		    rxvt_dbgmsg ((DBG_DEBUG, DBG_MENUBAR, "Got button release\n"));
 		    switch (item->entry.itemType)
 		    {
 			case MenuLabel:
@@ -1772,13 +1622,12 @@ rxvt_menu_select(rxvt_t *r, XButtonEvent *ev)
 			    }
 			    /* remove menu before sending keys to the application */
 			    rxvt_menu_hide_all(r);
-#ifndef DEBUG_MENU
+#ifndef DEBUG
 			    rxvt_dispatch_action(r, &(item->entry.action),
 				    (XEvent *) ev );
-#else		    /* DEBUG_MENU */
-			    fprintf(stderr, "%s: %s\n", item->name,
-				item->entry.action.str);
-#endif		    /* DEBUG_MENU */
+#else		    /* DEBUG */
+			    rxvt_dbgmsg ((DBG_DEBUG, DBG_MENUBAR, "%s: %s\n", item->name, item->entry.action.str));
+#endif		    /* DEBUG */
 			    break;
 		    }
 		    break;
@@ -1825,7 +1674,7 @@ rxvt_menubar_select(rxvt_t *r, XButtonEvent *ev)
 {
     menu_t	 *menu = NULL;
 
-    DBG_MSG( 2, (stderr, "rxvt_menubar_select()\n"));
+    rxvt_dbgmsg ((DBG_DEBUG, DBG_MENUBAR, "rxvt_menubar_select()\n"));
 
     /* determine the pulldown menu corresponding to the X index */
     if ( ev->y >= 0 && ev->y <= (rxvt_menubar_height(r)-MENUBAR_MARGIN) )
@@ -1904,7 +1753,7 @@ rxvt_menubar_create (rxvt_t* r)
     XGCValues	    gcvalue;
     unsigned long   gcmask;
 
-    DBG_MSG(2, (stderr,"rxvt_menubar_create\n"));
+    rxvt_dbgmsg ((DBG_DEBUG, DBG_MENUBAR,"rxvt_menubar_create\n"));
     /*
      * Only create a menubar if it's non-empty.
      */
@@ -2034,7 +1883,7 @@ rxvt_menubar_create (rxvt_t* r)
 void
 rxvt_menubar_clean_exit (rxvt_t* r)
 {
-    DBG_MSG( 2, (stderr, "rxvt_menubar_clean_exit()\n"));
+    rxvt_dbgmsg ((DBG_DEBUG, DBG_MENUBAR, "rxvt_menubar_clean_exit()\n"));
 
 #  ifdef XFT_SUPPORT
     /*
@@ -2072,7 +1921,7 @@ rxvt_menubar_clean_exit (rxvt_t* r)
 int
 rxvt_menubar_visible (rxvt_t* r)
 {
-    DBG_MSG( 3, (stderr, "rxvt_menubar_visible()\n"));
+    rxvt_dbgmsg ((DBG_DEBUG, DBG_MENUBAR, "rxvt_menubar_visible()\n"));
 
     return (IS_WIN(r->menuBar.win) && r->menuBar.state);
 }
@@ -2087,7 +1936,7 @@ rxvt_menubar_hide (rxvt_t* r)
 {
     int	    changed = 0;
 
-    DBG_MSG( 2, (stderr, "rxvt_menubar_hide()\n"));
+    rxvt_dbgmsg ((DBG_DEBUG, DBG_MENUBAR, "rxvt_menubar_hide()\n"));
 
     assert (IS_WIN(r->menuBar.win));
     changed = r->menuBar.state;
@@ -2107,7 +1956,7 @@ rxvt_menubar_show (rxvt_t* r)
 {
     int	    changed = 0;
 
-    DBG_MSG( 2, (stderr, "rxvt_menubar_show()\n"));
+    rxvt_dbgmsg ((DBG_DEBUG, DBG_MENUBAR, "rxvt_menubar_show()\n"));
     assert (IS_WIN(r->menuBar.win));
 
     changed = !r->menuBar.state;
@@ -2125,7 +1974,7 @@ rxvt_menubar_show (rxvt_t* r)
 void
 rxvt_menubar_expose(rxvt_t *r)
 {
-    DBG_MSG( 2, (stderr, "rxvt_menubar_expose()\n"));
+    rxvt_dbgmsg ((DBG_DEBUG, DBG_MENUBAR, "rxvt_menubar_expose()\n"));
 
     if (!r->menuBar.state || NOT_WIN(r->menuBar.win))
 	return;
@@ -2151,7 +2000,7 @@ rxvt_menubar_draw_labels( rxvt_t *r)
 
 	x = (menu->x + menu->len + HSPACE);
 
-# ifdef DEBUG_MENU_LAYOUT
+# ifdef DEBUG
 	rxvt_print_menu_descendants(menu);
 # endif
 
@@ -2310,7 +2159,7 @@ rxvt_menubar_dispatcher(rxvt_t *r, unsigned char *str)
     int		    n, cmd;
     unsigned char   *path, *name, *name2;
 
-    DBG_MSG( 2, (stderr, "rxvt_menubar_dispatcher()\n"));
+    rxvt_dbgmsg ((DBG_DEBUG, DBG_MENUBAR, "rxvt_menubar_dispatcher()\n"));
 
 #if 0
     if (rxvt_menubar_visible(r) && NOT_NULL(r->h->ActiveMenu))
@@ -2466,7 +2315,7 @@ rxvt_menubar_dispatcher(rxvt_t *r, unsigned char *str)
 			(name > path && name[-1] != '/')
 		       )
 		    {
-			rxvt_print_error("menu error <%s>\n", path);
+			rxvt_msg (DBG_ERROR, DBG_MENUBAR, "menu error <%s>\n", path);
 			break;
 		    }
 		    if (str[1] == MENUITEM_BEG)
@@ -2476,7 +2325,7 @@ rxvt_menubar_dispatcher(rxvt_t *r, unsigned char *str)
 
 			if (IS_NULL(str))
 			{
-			    rxvt_print_error("menu error <%s>\n", path);
+			    rxvt_msg (DBG_ERROR, DBG_MENUBAR, "menu error <%s>\n", path);
 			    break;
 			}
 			name2[-2] = '\0';   /* remove prev MENUITEM_END */
@@ -2490,13 +2339,8 @@ rxvt_menubar_dispatcher(rxvt_t *r, unsigned char *str)
 		    while (isspace((int) *str))
 			str++;	/* skip space */
 		}
-# ifdef DEBUG_MENU
-		fprintf(stderr,
-		    "`%c' path = <%s>, name = <%s>, name2 = <%s>, action = <%s>\n",
-		    cmd, (path ? path : "(nil)"), (name ? name : "(nil)"),
-		    (name2 ? name2 : "(nil)"), (str ? str : "(nil)")
-		);
-# endif
+
+		rxvt_dbgmsg ((DBG_DEBUG, DBG_MENUBAR, "`%c' path = <%s>, name = <%s>, name2 = <%s>, action = <%s>\n", cmd, (path ? (char*) path : "(nil)"), (name ? (char*) name : "(nil)"), (name2 ? (char*) name2 : "(nil)"), (str ? (char*) str : "(nil)")));
 	    }
 
 	    /* process the different commands */
@@ -2507,9 +2351,7 @@ rxvt_menubar_dispatcher(rxvt_t *r, unsigned char *str)
 		    {
 			int	     len;
 
-			DBG_MSG( 3, ( stderr, "Current menu %s\n",
-			    NOT_NULL(r->h->BuildMenu) ? r->h->BuildMenu->name :
-			    "(nil)"));
+			rxvt_dbgmsg ((DBG_DEBUG, DBG_MENUBAR, "Current menu %s\n", NOT_NULL(r->h->BuildMenu) ? (char *)r->h->BuildMenu->name : "(nil)"));
 
 			path = rxvt_menu_find_base(r, &(r->h->BuildMenu), path);
 			len = STRLEN(path);
@@ -2626,7 +2468,7 @@ rxvt_menubar_dispatcher(rxvt_t *r, unsigned char *str)
 void
 rxvt_menubar_control(rxvt_t *r, XButtonEvent *ev)
 {
-    DBG_MSG( 2, (stderr, "rxvt_menubar_control()\n"));
+    rxvt_dbgmsg ((DBG_DEBUG, DBG_MENUBAR, "rxvt_menubar_control()\n"));
 
     switch (ev->type)
     {
@@ -2690,13 +2532,13 @@ rxvt_menubar_load_file(rxvt_t *r, const unsigned char *filename)
     unsigned char   buffer[256];
     unsigned char  *p, *file, *tag = NULL;
 
-    DBG_MSG( 2, (stderr, "rxvt_menubar_load_file()\n"));
+    rxvt_dbgmsg ((DBG_DEBUG, DBG_MENUBAR, "rxvt_menubar_load_file()\n"));
 
     file = (unsigned char*) rxvt_File_find( (char*) filename,
 	    ".menu", r->h->rs[Rs_path]);
     if (IS_NULL(file))
     {
-	rxvt_print_error( "Could not open file %s", filename);
+	rxvt_msg (DBG_ERROR, DBG_MENUBAR,  "Could not open file %s", filename);
 	return;
     }
 
@@ -2713,11 +2555,9 @@ rxvt_menubar_load_file(rxvt_t *r, const unsigned char *filename)
 	    SET_NULL(tag);
     }
 
-# ifdef DEBUG_MENU
-    fprintf(stderr, "[read:%s]\n", filename);
+    rxvt_dbgmsg ((DBG_DEBUG, DBG_MENUBAR, "[read:%s]\n", filename));
     if (tag)
-	fprintf(stderr, "looking for [menu:%s]\n", tag);
-# endif
+	rxvt_dbgmsg ((DBG_DEBUG, DBG_MENUBAR, "looking for [menu:%s]\n", tag));
 
     while (NOT_NULL(p = (unsigned char*) fgets( (char*) buffer, sizeof(buffer), fp)))
     {
@@ -2734,9 +2574,7 @@ rxvt_menubar_load_file(rxvt_t *r, const unsigned char *filename)
 		    n += rxvt_str_match( (char*) p + n, (char*) tag);
 		    if (p[n] == ']')
 		    {
-# ifdef DEBUG_MENU
-			fprintf(stderr, "[menu:%s]\n", tag);
-# endif
+			rxvt_dbgmsg ((DBG_DEBUG, DBG_MENUBAR, "[menu:%s]\n", tag));
 			break;
 		    }
 		}
@@ -2752,9 +2590,7 @@ rxvt_menubar_load_file(rxvt_t *r, const unsigned char *filename)
     {
 	int	     n;
 
-# ifdef DEBUG_MENU
-	fprintf(stderr, "read line = %s\n", p);
-# endif
+	rxvt_dbgmsg ((DBG_DEBUG, DBG_MENUBAR, "read line = %s\n", p));
 
 	/* looking for [done:tag] or [done:] */
 	if ((n = rxvt_str_match( (char*) p, "[done")) != 0)
@@ -2775,9 +2611,7 @@ rxvt_menubar_load_file(rxvt_t *r, const unsigned char *filename)
 		    n += rxvt_str_match( (char*) p + n, (char*) tag);
 		    if (p[n] == ']')
 		    {
-# ifdef DEBUG_MENU
-			fprintf(stderr, "[done:%s]\n", tag);
-# endif
+			rxvt_dbgmsg ((DBG_DEBUG, DBG_MENUBAR, "[done:%s]\n", tag));
 			break;
 		    }
 		}
@@ -2809,7 +2643,7 @@ rxvt_menubar_load_file(rxvt_t *r, const unsigned char *filename)
 unsigned short
 rxvt_menubar_height(rxvt_t *r)
 {
-    DBG_MSG( 3, (stderr, "rxvt_menubar_height()\n"));
+    rxvt_dbgmsg ((DBG_DEBUG, DBG_MENUBAR, "rxvt_menubar_height()\n"));
 
     /* If menubar is not created or not mapped, return 0 */
     return  (NOT_WIN(r->menuBar.win) || !r->menuBar.state) ?
@@ -2821,7 +2655,7 @@ rxvt_menubar_height(rxvt_t *r)
 unsigned short
 rxvt_menubar_rheight(rxvt_t *r)
 {
-    DBG_MSG( 3, (stderr, "rxvt_menubar_rheight()\n"));
+    rxvt_dbgmsg ((DBG_DEBUG, DBG_MENUBAR, "rxvt_menubar_rheight()\n"));
 
 # ifdef XFT_SUPPORT
     if (ISSET_OPTION(r, Opt_xft))
@@ -2836,7 +2670,7 @@ rxvt_menubar_rheight(rxvt_t *r)
 int
 rxvt_is_menubar_win(rxvt_t *r, Window w)
 {
-    DBG_MSG( 3, (stderr, "rxvt_is_menubar_win()\n"));
+    rxvt_dbgmsg ((DBG_DEBUG, DBG_MENUBAR, "rxvt_is_menubar_win()\n"));
 
     return (w == r->menuBar.win);
 }
@@ -2848,7 +2682,7 @@ rxvt_menubar_resize(rxvt_t *r)
     menu_t  *menu;
     int	    i;
 
-    DBG_MSG( 2, (stderr, "rxvt_menubar_resize()\n"));
+    rxvt_dbgmsg ((DBG_DEBUG, DBG_MENUBAR, "rxvt_menubar_resize()\n"));
 
     if (IS_WIN(r->menuBar.win) && r->menuBar.state)
 	XMoveResizeWindow(r->Xdisplay, r->menuBar.win,
