@@ -48,6 +48,7 @@ void             rxvt_set_multichar_encoding      __PROTO((rxvt_t* r, const char
 #endif
 void             rxvt_decode_dummy                __PROTO((unsigned char* str, int len));
 void             rxvt_set_default_locale          __PROTO((rxvt_t *r));
+char*            rxvt_get_encoding_from_locale    __PROTO((rxvt_t *r));
 void             rxvt_set_default_font_x11        __PROTO((rxvt_t *r));
 char*            rxvt_fallback_mfont_x11          __PROTO((rxvt_t *r));
 #ifdef XFT_SUPPORT
@@ -96,7 +97,8 @@ void		 rxvt_copy_color                  __PROTO((rxvt_t*, int, int));
 void		 rxvt_set_color                   __PROTO((rxvt_t*, int, const XColor*));
 void             rxvt_init_env                    __PROTO((rxvt_t *r));
 void             rxvt_init_xlocale                __PROTO((rxvt_t *r));
-void             rxvt_init_command                __PROTO((rxvt_t* r, const char* const *argv));
+void		 rxvt_init_fifo	                  __PROTO((rxvt_t *r));
+void             rxvt_init_command                __PROTO((rxvt_t* r));
 CARD32           rxvt_get_desktop                 __PROTO((rxvt_t* r));
 void             rxvt_create_show_windows         __PROTO((rxvt_t* r, int argc, const char* const *argv));
 void             rxvt_destroy_termwin             __PROTO((rxvt_t* r, int page));
@@ -189,7 +191,6 @@ int		 rxvt_menu_select		  __PROTO((rxvt_t*, XButtonEvent*));
 
 /* Begin prototypes of misc.c */
 char           * rxvt_r_basename                  __PROTO((const char* str));
-void             rxvt_print_error                 __PROTO((const char* fmt,...));
 int              rxvt_str_match                   __PROTO((const char* s1, const char* s2));
 const char*      rxvt_str_skip_space              __PROTO((const char* str));
 char*            rxvt_str_trim                    __PROTO((char* str));
@@ -434,6 +435,7 @@ void             rxvt_kill_page                 __PROTO((rxvt_t*, short));
 void             rxvt_remove_page               __PROTO((rxvt_t*, short));
 void             rxvt_activate_page             __PROTO((rxvt_t*, short));
 void		 rxvt_tabbar_set_visible_tabs   __PROTO((rxvt_t*, Bool));
+void		 refresh_tabbar_tab		__PROTO((rxvt_t*, int));
 void             rxvt_tabbar_set_title          __PROTO((rxvt_t*, short, const unsigned char TAINTED *));
 void             rxvt_tabbar_resize             __PROTO((rxvt_t*));
 void             rxvt_tabbar_dispatcher         __PROTO((rxvt_t*, XButtonEvent*));
@@ -449,10 +451,12 @@ unsigned short   rxvt_tabbar_rheight            __PROTO((rxvt_t*));
 unsigned int     rxvt_tab_width                 __PROTO((rxvt_t*, const char*));
 int              rxvt_is_tabbar_win             __PROTO((rxvt_t*, Window));
 void             rxvt_tabbar_draw_buttons       __PROTO((rxvt_t*));
+void		 rxvt_adjust_fd_number	        __PROTO((rxvt_t* r));
 void             rxvt_tabbar_change_color       __PROTO((rxvt_t*, int, const char*));
 void             rxvt_tabbar_highlight_tab      __PROTO((rxvt_t*, short, Bool));
 void             rxvt_tabbar_move_tab           __PROTO((rxvt_t*, short));
-/* Begin prototypes of tabbar.c */
+void		 sync_tab_title		        __PROTO((rxvt_t*, int));
+/* End prototypes of tabbar.c */
 
 /* Begin prototypes of session.c */
 #ifdef HAVE_X11_SM_SMLIB_H
@@ -462,5 +466,10 @@ void            rxvt_session_exit              __PROTO((rxvt_t* r));
 #endif	/* HAVE_X11_SM_SMLIB_H */
 /* End prototypes of session.c */
 
+/* Begin prototypes of debug.c */
+int             rxvt_msg                       __PROTO((uint32_t, uint32_t, const char*, ...));
+void            rxvt_parse_dbg_arguments       __PROTO((int argc, const char* const*));
+/* End prototypes of debug.c */
+
 #endif  /* __PROTOS_H__ */
 /*----------------------- end-of-file (H source) -----------------------*/
