@@ -55,6 +55,7 @@
 
 
 #include <stdio.h>
+#include <stdint.h>
 #include <ctype.h>
 #include <errno.h>
 #include <signal.h>
@@ -83,10 +84,6 @@
 # include <string.h>
 #endif
 
-#ifdef HAVE_INTTYPES_H
-# include <inttypes.h>
-#endif
-
 #ifdef HAVE_LIMITS_H
 # include <limits.h>
 #endif
@@ -504,6 +501,7 @@ typedef char*	    XPointer;
 ** The following are internal includes
 *********************************************************
 */
+#include "debug.h"
 #include "encoding.h"
 #include "rxvtlib.h"
 
@@ -616,7 +614,7 @@ struct mouse_event {
     }
 
 #define BOUND_POSITIVE_INT16(val)				    \
-    (RINT16T)( (val) <= 0 ? 0 : min( (val), (((RUINT16T)-1)>>1) )  )
+    (int16_t)( (val) <= 0 ? 0 : min( (val), (((uint16_t)-1)>>1) )  )
 
 /*
  *****************************************************************************
@@ -967,6 +965,8 @@ enum {
 #define MRxvt_term	    (60) /* Change window title */
 #define MRxvt_tab	    (61) /* change tab title */
 #define MRxvt_tabterm	    (62) /* change tab and terminal title */
+#define MRxvt_wformat	    (63) /* Change window title format */
+#define MRxvt_tformat	    (64) /* Change tab title format */
 /* #define MRxvt_newtab	    (63) ** DISABLED create a new tab with title */
 /* #define MRxvt_prevtab    (64) ** DISABLED switch to previous tab */
 /* #define MRxvt_nexttab    (65) ** DISABLED switch to next tab */
@@ -1086,6 +1086,8 @@ enum colour_list {
  */
 enum {
     Rs_display_name = 0,
+    Rs_debug_masks,
+    Rs_debug_level,
     Rs_container_window,
     Rs_term_name,
     Rs_iconName,
@@ -1109,9 +1111,7 @@ enum {
 
 #ifdef BACKGROUND_IMAGE
     Rs_tabbarPixmap,	/* tabbar background pixmap */
-# if 0 /* appicon not yet implemented */
     Rs_appIcon,		/* use pixmap as application icon */
-# endif
 #endif
 
 #ifdef HAVE_MENUBAR
@@ -1231,6 +1231,8 @@ enum {
     Rs_holdExit,	_Rs_holdExit	= MAX_PROFILES - 1 + Rs_holdExit,
     Rs_holdExitTxt,	_Rs_holdExitTxt = MAX_PROFILES - 1 + Rs_holdExitTxt,
     Rs_holdExitTtl,	_Rs_holdExitTtl = MAX_PROFILES - 1 + Rs_holdExitTtl,
+    Rs_titleFormat,	_Rs_titleFormat = MAX_PROFILES - 1 + Rs_titleFormat,
+    Rs_winTitleFormat,	_Rs_winTitleFormat = MAX_NFONTS - 1 + Rs_winTitleFormat,
 
     NUM_RESOURCES
 } ;
@@ -1361,36 +1363,36 @@ enum {
  *****************************************************************************
  */
 
-/* convert pixel dimensions to row/column values.  Everything as RINT32T */
-#define Pixel2Width(x)	    ((RINT32T)(x) / (RINT32T)r->TermWin.fwidth)
-#define Pixel2Height(y)	    ((RINT32T)(y) / (RINT32T)r->TermWin.fheight)
-#define Width2Pixel(n)	    ((RINT32T)(n) * (RINT32T)r->TermWin.fwidth)
-#define Height2Pixel(n)	    ((RINT32T)(n) * (RINT32T)r->TermWin.fheight)
-#define Pixel2Col(x)	    Pixel2Width((RINT32T)(x) - (RINT32T)r->TermWin.int_bwidth)
-#define Pixel2Row(y)	    Pixel2Height((RINT32T)(y) - (RINT32T)r->TermWin.int_bwidth)
-#define Col2Pixel(col)	    ((RINT32T)Width2Pixel(col) + (RINT32T)r->TermWin.int_bwidth)
-#define Row2Pixel(row)	    ((RINT32T)Height2Pixel(row) + (RINT32T)r->TermWin.int_bwidth)
+/* convert pixel dimensions to row/column values.  Everything as int32_t */
+#define Pixel2Width(x)	    ((int32_t)(x) / (int32_t)r->TermWin.fwidth)
+#define Pixel2Height(y)	    ((int32_t)(y) / (int32_t)r->TermWin.fheight)
+#define Width2Pixel(n)	    ((int32_t)(n) * (int32_t)r->TermWin.fwidth)
+#define Height2Pixel(n)	    ((int32_t)(n) * (int32_t)r->TermWin.fheight)
+#define Pixel2Col(x)	    Pixel2Width((int32_t)(x) - (int32_t)r->TermWin.int_bwidth)
+#define Pixel2Row(y)	    Pixel2Height((int32_t)(y) - (int32_t)r->TermWin.int_bwidth)
+#define Col2Pixel(col)	    ((int32_t)Width2Pixel(col) + (int32_t)r->TermWin.int_bwidth)
+#define Row2Pixel(row)	    ((int32_t)Height2Pixel(row) + (int32_t)r->TermWin.int_bwidth)
 
 /*
-#define TermWin_TotalWidth()	((RINT32T)r->TermWin.width  + 2 * (RINT32T)r->TermWin.int_bwidth)
-#define TermWin_TotalHeight()	((RINT32T)r->TermWin.height + 2 * (RINT32T)r->TermWin.int_bwidth)
+#define TermWin_TotalWidth()	((int32_t)r->TermWin.width  + 2 * (int32_t)r->TermWin.int_bwidth)
+#define TermWin_TotalHeight()	((int32_t)r->TermWin.height + 2 * (int32_t)r->TermWin.int_bwidth)
 */
 
 /* how to build & extract colors and attributes */
 #define GET_BASEFG(x)	    (((x) & RS_fgMask))
 #define GET_BASEBG(x)	    (((x) & RS_bgMask)>>Color_Bits)
-#ifndef NO_BRIGHTCOLOR
-# define GET_FGCOLOR(x)			    \
-    ((((x) & RS_Bold) == 0		    \
-      || GET_BASEFG(x) < minCOLOR		\
-      || GET_BASEFG(x) >= minBrightCOLOR)	    \
-     ? GET_BASEFG(x)			    \
+#if 0 /* Was ifndef NO_BRIGHTCOLOR */
+# define GET_FGCOLOR(x)						    \
+    ((((x) & RS_Bold) == 0		    	    		    \
+      || GET_BASEFG(x) < minCOLOR		    		    \
+      || GET_BASEFG(x) >= minBrightCOLOR)	    		    \
+     ? GET_BASEFG(x)				    		    \
      : (GET_BASEFG(x) + (minBrightCOLOR - minCOLOR)))
-# define GET_BGCOLOR(x)			    \
-    ((((x) & RS_Blink) == 0		    \
-      || GET_BASEBG(x) < minCOLOR		\
-      || GET_BASEBG(x) >= minBrightCOLOR)	    \
-     ? GET_BASEBG(x)			    \
+# define GET_BGCOLOR(x)						    \
+    ((((x) & RS_Blink) == 0		    			    \
+      || GET_BASEBG(x) < minCOLOR				    \
+      || GET_BASEBG(x) >= minBrightCOLOR)			    \
+     ? GET_BASEBG(x)						    \
      : (GET_BASEBG(x) + (minBrightCOLOR - minCOLOR)))
 #else
 # define GET_FGCOLOR(x)	    GET_BASEFG(x)
@@ -1538,11 +1540,11 @@ struct rxvt_hidden {
     KeySym	    ks_greekmodeswith;
 #endif
 
-    RUINT16T	    prev_ncol,
+    uint16_t	    prev_ncol,
 		    prev_nrow;				/* screen: previous
 							   number of columns and
 							   rows */
-    RUINT32T	    pixcolor_set[NPIXCLR_SETS];
+    uint32_t	    pixcolor_set[NPIXCLR_SETS];
 
 #ifdef SELECTION_SCROLLING
     int		    scroll_selection_delay,
@@ -1655,10 +1657,6 @@ struct rxvt_hidden {
 #endif
     struct timeval  timeout[NUM_TIMEOUTS];
 
-    unsigned long   focusDelay;
-    struct timeval  lastFocusChange;			/* Time of the last
-							   FocusChange XEvent */
-
     /*
      * These three don't need to be kept but do so to placate some memory
      * checkers
@@ -1819,21 +1817,12 @@ struct rxvt_hidden {
 # define rxvt_set_window_color(r, page, idx, color) ()
 #endif
 
-#ifdef DEBUG_malloc
-# include "dmalloc.h"	    /* This comes last */
-#endif
-
-#ifdef DEBUG
-# define DEBUG_VERBOSE
-#endif
-
+/* This comes last */
 /*
- * Macro that always prints the debug messages. Useful if you want to just print
- * a few debug messages temporarily while hacking.
- */
-#define DBG_TMSG( d, x)					\
-    fprintf( stderr, "%s:%d ", __FILE__, __LINE__ );	\
-    fprintf x
+#ifdef DEBUG_malloc
+# include "dmalloc.h"
+#endif
+*/
 
 #endif		    /* __RXVT_H__ */
 /*----------------------- end-of-file (H source) -----------------------*/
