@@ -37,40 +37,8 @@
 #endif
 
 
-/*
- * If we haven't pulled in typedef's like  RINT16T  then do them ourself
- * type of (normal and unsigned) basic sizes
- */
-#if (SIZEOF_INT_P == 8)
-/* we have 8 byte pointers on 64bit systems */
-# if (SIZEOF_INT == 8)
-typedef int		intp_t;
-typedef unsigned int	u_intp_t;
-# elif (SIZEOF_LONG == 8)
-typedef long		intp_t;
-typedef unsigned long	u_intp_t;
-# else
-#  error No 8 byte integer type available
-# endif
-#else
-/*
- * If we have <inttypes.h>, use *intptr_t instead of *INT32T. This eliminates
- * some problems on 64-bit systems. Reported by David Mosberger
- * (http://bugs.debian.org/cgi-bin/bugreport.cgi?bug=312710). Patch is applied
- * after slight modification. :-)
- */
-# ifdef HAVE_INTTYPES_H
-typedef intptr_t	intp_t;
-typedef uintptr_t	u_intp_t;
-# else
-/* whatever normal size corresponds to a integer pointer */
-typedef RINT32T		intp_t;
-/* whatever normal size corresponds to a unsigned integer pointer */
-typedef RUINT32T	u_intp_t;
-# endif	/* HAVE_INTTYPES_H */
-#endif
 /* type of unicode_t */
-typedef RUINT32T	unicode_t;
+typedef uint32_t	unicode_t;
 
 
 /*****************************************************************************
@@ -82,45 +50,48 @@ struct rxvt_vars;	/* defined later on */
 struct rxvt_hidden;	/* not defined here */
 
 typedef struct {
-    RINT32T         row;
-    RINT32T         col;
+    int32_t         row;
+    int32_t         col;
 } row_col_t;
 
 typedef unsigned char text_t;
 #if defined(TTY_256COLOR) || defined(MULTICHAR_SET)
-# define rend_t	    RUINT32T
+# define rend_t	    uint32_t
 #else
-# define rend_t	    RUINT16T
+# define rend_t	    uint16_t
 #endif
 
+/* Size of the FIFO buffer */
+#define FIFO_BUF_SIZE (512)
+
 /*
  * TermWin elements limits
- *  ncol      : 1 <= ncol       <= MAX(RINT16T)
- *  nrow      : 1 <= nrow       <= MAX(RINT16T)
- *  saveLines : 0 <= saveLines  <= MAX(RINT16T)
+ *  ncol      : 1 <= ncol       <= MAX(int16_t)
+ *  nrow      : 1 <= nrow       <= MAX(int16_t)
+ *  saveLines : 0 <= saveLines  <= MAX(int16_t)
  *  nscrolled : 0 <= nscrolled  <= saveLines
  *  view_start: 0 <= view_start <= nscrolled
  */
 
 typedef struct {
-    RUINT16T	    fwidth,	/* font width  [pixels] */
+    uint16_t	    fwidth,	/* font width  [pixels] */
 		    fheight;	/* font height [pixels] */
 #ifdef XFT_SUPPORT
-    RUINT16T	    pwidth,	/* propotionally spaced font width / height */
+    uint16_t	    pwidth,	/* propotionally spaced font width / height */
 		    pheight;
 #endif
-    RUINT16T	    propfont;	/* font proportional flags */
-    RUINT16T	    ncol;	/* window columns [characters] */
-    RUINT16T	    nrow;	/* window rows [characters] */
-    RUINT16T	    mapped; 	/* TermWin is mapped? */
-    RUINT16T	    int_bwidth; /* internal border width */
-    RUINT16T	    ext_bwidth; /* external border width */
+    uint16_t	    propfont;	/* font proportional flags */
+    uint16_t	    ncol;	/* window columns [characters] */
+    uint16_t	    nrow;	/* window rows [characters] */
+    uint16_t	    mapped; 	/* TermWin is mapped? */
+    uint16_t	    int_bwidth; /* internal border width */
+    uint16_t	    ext_bwidth; /* external border width */
 
-    RUINT16T	    maxTabWidth,    /* max width of tab title to display */
+    uint16_t	    maxTabWidth,    /* max width of tab title to display */
 		    minVisibleTabs; /* Minimum number of tabs to try and keep
 				       visible */
 #ifndef NO_LINESPACE
-    RUINT16T	    lineSpace;	/* space between rows */
+    uint16_t	    lineSpace;	/* space between rows */
 #endif
 
     char	    BOOLVAR(focus,1);	/* window is focused? */
@@ -232,15 +203,15 @@ typedef struct {
  */
 typedef struct {
     text_t**	    text;	/* _all_ the text */
-    RINT16T*	    tlen;	/* length of each text line */
+    int16_t*	    tlen;	/* length of each text line */
     rend_t**	    rend;	/* rendition, uses RS_ flags */
     row_col_t       cur;	/* cursor position on the screen */
-    RUINT16T	    tscroll;	/* top of settable scroll region */
-    RUINT16T	    bscroll;	/* bottom of settable scroll region */
-    RUINT16T	    charset;	/* character set number [0..3] */
+    uint16_t	    tscroll;	/* top of settable scroll region */
+    uint16_t	    bscroll;	/* bottom of settable scroll region */
+    uint16_t	    charset;	/* character set number [0..3] */
     unsigned int    flags;	/* see below */
     row_col_t	    s_cur;	/* saved cursor position */
-    RUINT16T	    s_charset;	/* saved character set number [0..3] */
+    uint16_t	    s_charset;	/* saved character set number [0..3] */
     char	    s_charset_char;
     rend_t	    s_rstyle;	/* saved rendition style */
 } screen_t;
@@ -248,7 +219,7 @@ typedef struct {
 
 typedef struct {
     unsigned char*  text;   /* selected text */
-    RUINT32T	    len;    /* length of selected text */
+    uint32_t	    len;    /* length of selected text */
     enum {
 	SELECTION_CLEAR = 0,/* nothing selected */
 	SELECTION_INIT,	    /* marked a point */
@@ -309,7 +280,7 @@ typedef enum {
 #define Opt_forceTransparent	    ((1LU<<16) | IS_OPTION1)
 #define Opt_mc_hack		    ((1LU<<17) | IS_OPTION1)
 #define Opt_tripleclickwords	    ((1LU<<18) | IS_OPTION1)
-#define Opt_scrollWithBuffer	    ((1LU<<19) | IS_OPTION1)
+/* #define Opt_scrollWithBuffer	    ((1LU<<19) | IS_OPTION1) */
 #define Opt_mouseWheelScrollPage    ((1LU<<21) | IS_OPTION1)
 #define Opt_pointerBlank	    ((1LU<<22) | IS_OPTION1)
 #define Opt_cursorBlink		    ((1LU<<23) | IS_OPTION1)
@@ -325,6 +296,9 @@ typedef enum {
 #ifdef XFT_SUPPORT
 # define Opt_xft		    ((1LU<<29) | IS_OPTION1)
 #endif
+#ifdef USE_FIFO
+# define Opt_useFifo		    ((1LU<<30) | IS_OPTION1)
+#endif/*USE_FIFO*/
 #define DEFAULT_OPTIONS	    \
     (Opt_scrollBar)
 
@@ -347,22 +321,25 @@ typedef enum {
 #define Opt2_bottomTabbar	    ((1LU<<13) | IS_OPTION2)
 #define Opt2_borderLess		    ((1LU<<14) | IS_OPTION2)
 #define Opt2_overrideRedirect	    ((1LU<<15) | IS_OPTION2)
-/* #define Opt2_holdExit		    ((1LU<<16) | IS_OPTION2) */
-#define Opt2_broadcast		    ((1LU<<17) | IS_OPTION2)
-#define Opt2_hideButtons	    ((1LU<<18) | IS_OPTION2)
-#define Opt2_veryBold		    ((1LU<<19) | IS_OPTION2)
-#define Opt2_noSysConfig	    ((1LU<<20) | IS_OPTION2)
-#define Opt2_disableMacros	    ((1LU<<21) | IS_OPTION2)
-#ifdef HAVE_X11_SM_SMLIB_H
-# define Opt2_enableSessionMgt	    ((1LU<<22) | IS_OPTION2)
+#define Opt2_broadcast		    ((1LU<<16) | IS_OPTION2)
+#define Opt2_hideButtons	    ((1LU<<17) | IS_OPTION2)
+#define Opt2_veryBold		    ((1LU<<18) | IS_OPTION2)
+#ifndef NO_BRIGHTCOLOR
+# define Opt2_boldColors	    ((1LU<<19) | IS_OPTION2)
+# define Opt_veryBright		    ((1LU<<20) | IS_OPTION2)
 #endif
-#define Opt2_linuxHomeEndKey	    ((1LU<<23) | IS_OPTION2)
-#define Opt2_hlTabOnBell	    ((1LU<<24) | IS_OPTION2)
-#define Opt2_smoothResize	    ((1LU<<25) | IS_OPTION2)
-#define Opt2_smartResize	    ((1LU<<26) | IS_OPTION2)
-#define Opt2_autohideTabbar	    ((1LU<<27) | IS_OPTION2)
-#define Opt2_maximized		    ((1LU<<28) | IS_OPTION2)
-#define Opt2_fullscreen		    ((1LU<<29) | IS_OPTION2)
+#define Opt2_noSysConfig	    ((1LU<<21) | IS_OPTION2)
+#define Opt2_disableMacros	    ((1LU<<22) | IS_OPTION2)
+#ifdef HAVE_X11_SM_SMLIB_H
+# define Opt2_enableSessionMgt	    ((1LU<<23) | IS_OPTION2)
+#endif
+#define Opt2_linuxHomeEndKey	    ((1LU<<24) | IS_OPTION2)
+#define Opt2_hlTabOnBell	    ((1LU<<25) | IS_OPTION2)
+#define Opt2_smoothResize	    ((1LU<<26) | IS_OPTION2)
+#define Opt2_smartResize	    ((1LU<<27) | IS_OPTION2)
+#define Opt2_autohideTabbar	    ((1LU<<28) | IS_OPTION2)
+#define Opt2_maximized		    ((1LU<<29) | IS_OPTION2)
+#define Opt2_fullscreen		    ((1LU<<30) | IS_OPTION2)
 
 
 #ifdef XFT_SUPPORT
@@ -602,11 +579,11 @@ typedef struct {
 				       during this processes lifetime */
 
     /* moved from TermWin_t */
-    RUINT16T	    saveLines;	/* number of lines to save */
-    RUINT16T	    num_scr;	/* number of lines scrolled */
-    RUINT16T	    nscrolled;	/* number of line actually scrolled */
-    RUINT16T	    view_start;	/* scrollback view starts here */
-    RUINT16T	    mapped;	/* window state mapped? */
+    uint16_t	    saveLines;	/* number of lines to save */
+    uint16_t	    num_scr;	/* number of lines scrolled */
+    uint16_t	    nscrolled;	/* number of line actually scrolled */
+    uint16_t	    view_start;	/* scrollback view starts here */
+    uint16_t	    mapped;	/* window state mapped? */
 
     /* screen structure initialized? */
     unsigned char   BOOLVAR(init_screen, 1);
@@ -645,11 +622,16 @@ typedef struct {
     /* move from hidden */
     rend_t	    rstyle;
 
-    RUINT16T	    prev_ncol; /* previous columns */
-    RUINT16T	    prev_nrow; /* previous rows */
+    uint16_t	    prev_ncol; /* previous columns */
+    uint16_t	    prev_nrow; /* previous rows */
     /* moved from tab_t */
-    short		tab_width;  /* tab width */
-    char UNTAINTED *	tab_title;  /* tab title */
+    short		tab_width;	/* tab width */
+    char UNTAINTED *	tab_title;  	/* tab title */
+
+    char	    *title_format;	/* Format to be used to display the tab
+					   title */
+    char	    *winTitleFormat;	/* Format of the window title (used when
+					   syncing the tab title */
 
     /* moved from rxvt_t */
     int             cmd_fd; /* pty file descriptor; connected to command */
@@ -670,8 +652,8 @@ typedef struct {
 #endif
 
     /* moved from hidden */
-    RUINT32T	    PrivateModes;
-    RUINT32T	    SavedModes;
+    uint32_t	    PrivateModes;
+    uint32_t	    SavedModes;
 
 #ifdef UTMP_SUPPORT
 #ifndef UTEMPTER_SUPPORT
@@ -715,6 +697,8 @@ typedef struct {
     unsigned char   BOOLVAR(hold,2),
 		    BOOLVAR(holdOption,3),  /* Copy of the profile option for
 					       this tab.*/
+		    BOOLVAR(gotEIO,1),	    /* Read on this terminal's fd
+					       got EIO */
 		    BOOLVAR(dead,1),	    /* the terminal is dead or alive? */
 		    BOOLVAR(highlight,1);   /* the terminal is highlighted? */
     int		    status;		    /* Status of child process after it
@@ -802,11 +786,15 @@ enum {
     MacroFnToggleSubwin,
     MacroFnFont,
     MacroFnToggleVeryBold,
+    MacroFnToggleBoldColors,
+    MacroFnToggleVeryBright,
     MacroFnToggleTransp,
     MacroFnToggleBcst,
     MacroFnToggleHold,
-    MacroFnToggleFullscren,
+    MacroFnToggleFullscreen,
+    MacroFnRaise,
     MacroFnSetTitle,
+    MacroFnUseFifo,
     MacroFnPrintScreen,
     MacroFnSaveConfig,
     MacroFnToggleMacros,
@@ -880,7 +868,7 @@ typedef struct rxvt_vars {
 #endif
     tabBar_t	    tabBar;
     Display*	    Xdisplay;
-    RUINT32T	    Options[MAX_OPTION_ARRAY];
+    uint32_t	    Options[MAX_OPTION_ARRAY];
     XSizeHints      szHint;
 
     /* macros */
@@ -932,13 +920,14 @@ typedef struct rxvt_vars {
     short	    tabClicked;		    /* Tab clicked by user. Used for
 					       moving tabs by drag and drop. */
 
-    unsigned char   BOOLVAR( cleanDeadChilds, 1 );
+    unsigned char   BOOLVAR( cleanDeadChilds, 1 ),
 					    /* True if we have marked some
 					       children as dead, but not called
 					       rxvt_clean_cmd_page() to clean
 					       them out */
+		    BOOLVAR( gotEIO, 1 );   /* True if some read() got EIO */
 
-    short	    ndead_childs;		    /* number of children that have
+    short	    ndead_childs;	    /* number of children that have
 					       died */
     short	    nAsyncChilds;	    /* Number of alive children launched
 					       via rxvt_async_exec */
@@ -948,6 +937,17 @@ typedef struct rxvt_vars {
 
     int		    num_fds;		    /* number of fd to monitor */
 
+
+#ifdef USE_FIFO
+    int		    fifo_fd;			/* fd to read macros from */
+    char	    fifo_buf[FIFO_BUF_SIZE];	/* Buffer size of data read from
+						   fifo_fd */
+    char	    *fbuf_ptr;			/* Pointer where data from
+						   fifo_fd should be read */
+    char	    *fifo_name;			/* Name of the fifo */
+#endif
+
+
     selection_t     selection;
     sstyle_t	    selection_style;
 
@@ -957,6 +957,7 @@ typedef struct rxvt_vars {
 
     char**	    global_argv;
     int		    global_argc;
+
 } rxvt_t;
 
 
