@@ -50,18 +50,6 @@
 #endif /* HAVE_LIBXPM */
 
 
-#ifdef DEBUG_VERBOSE
-#define DEBUG_LEVEL 1
-#else 
-#define DEBUG_LEVEL 0
-#endif
-
-#if DEBUG_LEVEL
-#define DBG_MSG(d,x) if(d <= DEBUG_LEVEL) fprintf x
-#else
-#define DBG_MSG(d,x)
-#endif
-
 #ifdef XFT_SUPPORT
 # define    FHEIGHT pheight
 # define    FWIDTH  pwidth
@@ -134,7 +122,7 @@ static char** xpm_d_name[] =
     left_d_xpm,right_d_xpm
 };
 #else
-static char *xpm_name[] =
+static unsigned char *xbm_name[] =
 {
     term_bits,close_term_bits,
     left_bits,right_bits
@@ -375,8 +363,8 @@ draw_string (rxvt_t* r, Region clipRegion,
 		char*		ostr = str;
 
 		/* convert to UTF-8 */
-		iconv (r->TermWin.xfticonv, &ostr, (size_t*) &olen,
-		    &pstr, (size_t*) &plen);
+		iconv (r->TermWin.xfticonv, (char**) &ostr,
+		    (size_t*) &olen, &pstr, (size_t*) &plen);
 		*pstr = (char) 0;   /* set end of string */
 
 		rxvt_draw_string_xft (r, r->tabBar.win, r->tabBar.gc,
@@ -396,7 +384,7 @@ draw_string (rxvt_t* r, Region clipRegion,
 	    else
 #  endif
 	    {
-		DBG_MSG(1, (stderr, "XFT non-iconv tab title\n"));
+		rxvt_dbgmsg ((DBG_VERBOSE, DBG_TABBAR, "XFT non-iconv tab title\n"));
 		rxvt_draw_string_xft (r, r->tabBar.win, r->tabBar.gc,
 			clipRegion, RS_None,
 			active ? USE_BOLD_PFONT : USE_PFONT,
@@ -477,8 +465,7 @@ draw_string (rxvt_t* r, Region clipRegion,
  */
 /* INTPROTO */
 static void
-draw_title (rxvt_t* r, const char* orgstr, int x, int y, int tnum,
-	Region region)
+draw_title (rxvt_t* r, int x, int y, int tnum, Region region)
 {
     Region	clipRegion;
     char	str[MAX_DISPLAY_TAB_TXT + 1];
@@ -497,7 +484,7 @@ draw_title (rxvt_t* r, const char* orgstr, int x, int y, int tnum,
      * Adjust y offset, and make sure output is restricted to the current tab
      * title.
      */
-# ifdef XFT_SUPPORT
+#ifdef XFT_SUPPORT
     if (ISSET_OPTION(r, Opt_xft) && (NULL != r->tabBar.xftwin))
     {
 	if( r->TermWin.xftpfont )
@@ -527,14 +514,32 @@ draw_title (rxvt_t* r, const char* orgstr, int x, int y, int tnum,
 	else y -= r->TermWin.xftfont->descent;
     }
     else
-# endif
+#endif /* XFT_SUPPORT */
 	y -= r->TermWin.font->descent;
 
     /*
-     * Copy the title into str, and null terminate.
+     * Get the title into str. Under Xft, we use the format specified by
+     * title_format.
      */
-    STRNCPY (str, orgstr, r->TermWin.maxTabWidth);
-    str[r->TermWin.maxTabWidth] = (char) 0;
+#ifdef XFT_SUPPORT
+    if(
+	  NOTSET_OPTION( r, Opt_xft )			||
+	  IS_NULL( r->TermWin.xftpfont )		||
+	  IS_NULL( PVTS(r, tnum)->title_format )   	||
+	  rxvt_percent_interpolate( r, tnum,
+		PVTS(r, tnum)->title_format,
+		STRLEN( PVTS(r, tnum)->title_format ),
+		str, r->TermWin.maxTabWidth ) <= 1
+      )
+#endif /* XFT_SUPPORT */
+    {
+	/*
+	 * If % interpolation was not possible, or returned a 1 byte long
+	 * string, then just copy the title over.
+	 */
+	STRNCPY( str, PVTS(r,tnum)->tab_title , r->TermWin.maxTabWidth );
+	str[r->TermWin.maxTabWidth] = '\0';
+    }
 
 
     /*
@@ -640,15 +645,16 @@ draw_title (rxvt_t* r, const char* orgstr, int x, int y, int tnum,
     point.y	    = (short) ay
 
 /*
- * Refresh tab number page.
+ * Refresh title of tab "page"
  */
+/* EXTPROTO */
 void
 refresh_tabbar_tab( rxvt_t *r, int page)
 {
     int		i;
     XRectangle	rect;
 
-    DBG_MSG( 2, ( stderr, "Refreshing tabbar title of page %d\n", page));
+    rxvt_dbgmsg ((DBG_DEBUG, DBG_TABBAR, "Refreshing tabbar title of page %d\n", page));
 
     if( page < FVTAB(r) || page > LVTAB(r) ) return;
 
@@ -830,8 +836,8 @@ void rxvt_draw_tabs (rxvt_t* r, Region region)
 
 	    /* Draw the tab title. */
 	    CHOOSE_GC_FG( r, r->tabBar.fg);
-	    draw_title (r, PVTS(r, page)->tab_title,
-		    x + TXT_XOFF, ATAB_EXTRA / 2 + TXT_YOFF, page, region);
+	    draw_title (r, x + TXT_XOFF, ATAB_EXTRA / 2 + TXT_YOFF,
+		    page, region);
 	}
 
 	else /* if( page == ATAB(r) ) */
@@ -901,8 +907,7 @@ void rxvt_draw_tabs (rxvt_t* r, Region region)
 
 	    /* Choose GC foreground for tab title. */
 	    CHOOSE_GC_FG( r, r->tabBar.ifg);
-	    draw_title (r, PVTS(r, page)->tab_title,
-		    x + TXT_XOFF,
+	    draw_title (r, x + TXT_XOFF,
 		    ISSET_OPTION(r, Opt2_bottomTabbar) ?
 		     	    TXT_YOFF : ATAB_EXTRA + TXT_YOFF,
 		    page, region);
@@ -1125,35 +1130,34 @@ rxvt_kill_page (rxvt_t* r, short page)
 /*
  * Reduce r->num_fds so that select() is more efficient
  */
-/* INTPROTO */
+/* EXTPROTO */
 void
-rxvt_adjust_fd_number (rxvt_t* r)
+rxvt_adjust_fd_number( rxvt_t* r )
 {
-    int	    num_fds;
+    int	    num_fds = STDERR_FILENO;
+    int	    i;
 
+    for( i=0; i <= LTAB(r); i++ )
+	MAX_IT( num_fds, PVTS(r, i)->cmd_fd );
+    rxvt_dbgmsg ((DBG_DEBUG, DBG_TABBAR, "LTAB=%d, stderr_fd=%d, num_fds=%d. ", LTAB(r), STDERR_FILENO, num_fds));
 
-    num_fds = max( STDERR_FILENO, LVTS(r)->cmd_fd );
-
-    /* adjust num_fds based on X connection. notice that we do not
-     * need to check if Xfd is a valid file descriptor here because
-     * -1 is always smaller than positive number :-)
-     */
     MAX_IT( num_fds, r->Xfd );
+#ifdef USE_FIFO
+    MAX_IT( num_fds, r->fifo_fd );
+#endif/*USE_FIFO*/
 #ifdef HAVE_X11_SM_SMLIB_H
-    /* adjust num_fds based on ICE connection. notice that we do not
-     * need to check if ice_fd is a valid file descriptor here because
-     * -1 is always smaller than positive number :-)
-     */
     MAX_IT( num_fds, r->TermWin.ice_fd );
 #endif
 
+#if 0
     MAX_IT( num_fds, r->num_fds-1 );
+#endif
 #ifdef OS_IRIX
     /* Alex Coventry says we need 4 & 7 too */
     MAX_IT( num_fds, 7 );
 #endif
     r->num_fds = num_fds + 1;	/* counts from 0 */
-    DBG_MSG(1, (stderr, "Adjust num_fds to %d\n", r->num_fds));
+    rxvt_dbgmsg ((DBG_VERBOSE, DBG_TABBAR, "Adjust num_fds to %d\n", r->num_fds));
 }
 
 
@@ -1170,28 +1174,26 @@ rxvt_append_page( rxvt_t* r, int profile,
     char**  argv;
 
 
-    DBG_MSG( 2, ( stderr, "rxvt_append_page( r, %d, %s, %s )\n",
-		profile, title ? title : "(nil)",
-		command ? command : "(nil)" ) );
+    rxvt_dbgmsg ((DBG_DEBUG, DBG_TABBAR, "rxvt_append_page( r, %d, %s, %s )\n", profile, title ? title : "(nil)", command ? command : "(nil)" ));
 
     /* Sanitization */
     assert( LTAB(r) < MAX_PAGES );
 
     if (LTAB(r) == MAX_PAGES-1)
     {
-	rxvt_print_error( "Too many tabs" );
+	rxvt_msg (DBG_ERROR, DBG_TABBAR,  "Too many tabs" );
 	return ;
     }
 
     if( profile < 0 || profile >= MAX_PROFILES )
     {
-	rxvt_print_error( "Warning: Profile %d out of range", profile );
+	rxvt_msg (DBG_WARN, DBG_TABBAR,  "Warning: Profile %d out of range", profile );
 	profile = 0;
     }
 
     /* indicate that we add a new tab */
     LTAB(r)++;
-    DBG_MSG( 1, ( stderr, "last page is %d\n", LTAB(r)) );
+    rxvt_dbgmsg ((DBG_VERBOSE, DBG_TABBAR, "last page is %d\n", LTAB(r)));
 
     /*
      * Use command specified with -e only if we're opening the first tab, or the
@@ -1227,9 +1229,7 @@ rxvt_append_page( rxvt_t* r, int profile,
 	else
 	    argv = NULL;
     }
-    DBG_MSG( 2, ( stderr, "Forking command=%s, argv[0]=%s\n",
-		command ? command : "(nil)",
-		( argv && argv[0] ) ? argv[0] : "(nil)" ) );
+    rxvt_dbgmsg ((DBG_DEBUG, DBG_TABBAR, "Forking command=%s, argv[0]=%s\n", command ? command : "(nil)", ( argv && argv[0] ) ? argv[0] : "(nil)" ));
 
     /*
      * Set the tab title.
@@ -1292,8 +1292,7 @@ rxvt_append_page( rxvt_t* r, int profile,
 	if( len > 0 && chdir( child_cwd ) == 0 )
 	{
 	    /* Now in working directory of ATAB */
-	    DBG_MSG( 2, ( stderr, "Running child in directory: %s\n",
-			child_cwd ));
+	    rxvt_dbgmsg ((DBG_DEBUG, DBG_TABBAR, "Running child in directory: %s\n", child_cwd ));
 
 	    /* Run command in this new directory. */
 	    LVTS(r)->cmd_fd =
@@ -1306,7 +1305,7 @@ rxvt_append_page( rxvt_t* r, int profile,
 	else
 	{
 	    /* Exec command in original directory. */
-	    DBG_MSG( 2, ( stderr, "Running child in original directory\n"));
+	    rxvt_dbgmsg ((DBG_DEBUG, DBG_TABBAR, "Running child in original directory\n"));
 
 	    LVTS(r)->cmd_fd =
 		rxvt_run_command( r, LTAB(r), (const char**) argv );
@@ -1338,7 +1337,7 @@ rxvt_append_page( rxvt_t* r, int profile,
 	LTAB(r) --;
 	return;
     }
-    DBG_MSG(2,(stderr,"page %d's cmd_fd is %d\n", LTAB(r), LVTS(r)->cmd_fd));
+    rxvt_dbgmsg ((DBG_DEBUG, DBG_TABBAR,"page %d's cmd_fd is %d\n", LTAB(r), LVTS(r)->cmd_fd));
 
 
     /* adjust number of file descriptors to listen */
@@ -1392,8 +1391,7 @@ rxvt_append_page( rxvt_t* r, int profile,
 
     /* synchronize terminal title with tab title */
     if (ISSET_OPTION(r, Opt2_syncTabTitle))
-	rxvt_set_term_title (r,
-		(const unsigned char*) PVTS(r, ATAB(r))->tab_title);
+	sync_tab_title( r, ATAB(r) );
 
     /* synchronize icon name to tab title */
     if (ISSET_OPTION(r, Opt2_syncTabIcon))
@@ -1412,7 +1410,7 @@ rxvt_remove_page (rxvt_t* r, short page)
     register int    i;
 
 
-    DBG_MSG(1, (stderr,"remove_page(%d)\n", page));
+    rxvt_dbgmsg ((DBG_VERBOSE, DBG_TABBAR,"remove_page(%d)\n", page));
 
 
     /* clean utmp/wtmp entry */
@@ -1434,13 +1432,6 @@ rxvt_remove_page (rxvt_t* r, short page)
 	PVTS(r, page)->v_buffer = NULL;
     }
 
-    /* to adjust num_fds if necessary */
-    if (PVTS(r, page)->cmd_fd == r->num_fds-1)
-    {
-	r->num_fds --;
-	DBG_MSG(1, (stderr, "Adjust num_fds to %d\n", r->num_fds));
-    }
-
     /* free screen structure */
     rxvt_scr_release (r, page);
 
@@ -1491,6 +1482,10 @@ rxvt_remove_page (rxvt_t* r, short page)
     /* always set mapped flag */
     AVTS(r)->mapped = 1;
 
+    /* Adjust the number of FD's we select() for.  */
+    rxvt_adjust_fd_number(r);
+
+
     /* adjust visible tabs */
     rxvt_tabbar_set_visible_tabs (r, True);
     refresh_tabbar_tab( r, ATAB(r));	/* Active tab has changed */
@@ -1520,11 +1515,12 @@ rxvt_remove_page (rxvt_t* r, short page)
 
     /* synchronize terminal title with tab title */
     if (ISSET_OPTION(r, Opt2_syncTabTitle))
-	rxvt_set_term_title (r, (const unsigned char*) PVTS(r, ATAB(r))->tab_title);
+	sync_tab_title( r, ATAB(r) );
 
     /* synchronize icon name to tab title */
     if (ISSET_OPTION(r, Opt2_syncTabIcon))
-	rxvt_set_icon_name(r, (const unsigned char*) PVTS(r, ATAB(r))->tab_title);
+	rxvt_set_icon_name(r,
+		(const unsigned char*) PVTS(r, ATAB(r))->tab_title);
 }
 
 
@@ -1567,8 +1563,7 @@ rxvt_tabbar_set_title (rxvt_t* r, short page, const unsigned char TAINTED * str)
     /* synchronize terminal title with active tab title */
     if (ISSET_OPTION(r, Opt2_syncTabTitle) &&
 	(page == ATAB(r)))
-	rxvt_set_term_title (r,
-		(const unsigned char*) PVTS(r, ATAB(r))->tab_title);
+	sync_tab_title( r, ATAB(r) );
 
     /* synchronize icon name to tab title */
     if (ISSET_OPTION(r, Opt2_syncTabIcon) &&
@@ -1619,12 +1614,11 @@ rxvt_activate_page (rxvt_t* r, short index)
      * under slow connections).
      */
     /* rxvt_scr_touch (r, ATAB(r), True); */
-    DBG_MSG( 1, (stderr, "active page is %d\n",ATAB(r)) );
+    rxvt_dbgmsg ((DBG_VERBOSE, DBG_TABBAR, "active page is %d\n",ATAB(r)));
 
     /* synchronize terminal title with tab title */
     if (ISSET_OPTION(r, Opt2_syncTabTitle))
-	rxvt_set_term_title (r,
-		(const unsigned char*) PVTS(r, ATAB(r))->tab_title);
+	sync_tab_title( r, ATAB(r) );
 
     /* synchronize icon name to tab title */
     if (ISSET_OPTION(r, Opt2_syncTabIcon))
@@ -1678,7 +1672,7 @@ rxvt_tabbar_dispatcher (rxvt_t* r, XButtonEvent* ev)
     x = ev->x;
     y = ev->y;
     but = -1;
-    DBG_MSG( 2, ( stderr, "click in (%d,%d)\n", x, y));
+    rxvt_dbgmsg ((DBG_DEBUG, DBG_TABBAR, "click in (%d,%d)\n", x, y));
 
     /* Button4/Button5 of wheel mouse activate the left/right tab as
        Mozilla firefox */
@@ -1739,11 +1733,11 @@ rxvt_tabbar_dispatcher (rxvt_t* r, XButtonEvent* ev)
 	/* we should only handle left-mouse-button clicks */
 	if ( ev->button != Button1 )
 	{
-	    DBG_MSG(1,(stderr,"skip non-left-mouse-button click\n"));
+	    rxvt_dbgmsg ((DBG_VERBOSE, DBG_TABBAR,"skip non-left-mouse-button click\n"));
 	    return;
 	}
 
-	DBG_MSG(1,(stderr,"click on button %d\n",but));
+	rxvt_dbgmsg ((DBG_VERBOSE, DBG_TABBAR,"click on button %d\n",but));
 	switch(but)
 	{
 	    case 0 : /* right shift */
@@ -1793,7 +1787,7 @@ rxvt_tabbar_dispatcher (rxvt_t* r, XButtonEvent* ev)
 	{
 	    but = i - 1;
 
-	    DBG_MSG( 2, ( stderr,"click on tab %d\n", but));
+	    rxvt_dbgmsg ((DBG_DEBUG, DBG_TABBAR,"click on tab %d\n", but));
 	    switch( ev->button )
 	    {
 		case Button1:
@@ -1853,8 +1847,7 @@ rxvt_tabbar_button_release( rxvt_t *r, XButtonEvent *ev)
 	    )
 	    w += TAB_WIDTH( droppedTab );
 
-	DBG_MSG( 2, ( stderr, "Dragged tab %d to %d (%d, %d)\n",
-		r->tabClicked, droppedTab - 1, ev->x, ev->y) );
+	rxvt_dbgmsg ((DBG_DEBUG, DBG_TABBAR, "Dragged tab %d to %d (%d, %d)\n", r->tabClicked, droppedTab - 1, ev->x, ev->y));
 
 	/* Move active tab there */
 	rxvt_tabbar_move_tab( r, droppedTab - 1 );
@@ -1975,7 +1968,7 @@ rxvt_tabbar_create (rxvt_t* r)
 
 
     init_tabbar (r);
-    DBG_MSG(1,(stderr,"Creating tabbar\n"));
+    rxvt_dbgmsg ((DBG_VERBOSE, DBG_TABBAR,"Creating tabbar\n"));
 
 
     /* initialize the colors */
@@ -2092,8 +2085,7 @@ rxvt_tabbar_create (rxvt_t* r)
 	else
 	    r->tabBar.delimit = VTFG(r,0);
 
-	DBG_MSG( 2, (stderr, "Delimit color: %hx, %hx, %hx (#%lx)\n",
-		color.red, color.green, color.blue, r->tabBar.delimit));
+	rxvt_dbgmsg ((DBG_DEBUG, DBG_TABBAR, "Delimit color: %hx, %hx, %hx (#%lx)\n", color.red, color.green, color.blue, r->tabBar.delimit));
     }
 
 
@@ -2234,15 +2226,13 @@ rxvt_tabbar_create (rxvt_t* r)
 	assert (IS_PIXMAP(img_d[i]));
 #else
 	img[i] = XCreatePixmapFromBitmapData (r->Xdisplay,
-	    r->tabBar.win, xpm_name[i], BTN_WIDTH, BTN_HEIGHT,
+	    r->tabBar.win, (char *) xbm_name[i], BTN_WIDTH, BTN_HEIGHT,
 	    r->tabBar.fg, r->tabBar.bg, XDEPTH);
 	assert (IS_PIXMAP(img[i]));
 #endif
     }
 
-    DBG_MSG( 3, ( stderr,
-		"TXT_XOFF=%d, TXT_YOFF=%d, ATAB_EXTRA=%d, TAB_RADIUS=%d\n",
-		TXT_XOFF, TXT_YOFF, ATAB_EXTRA, TAB_RADIUS) );
+    rxvt_dbgmsg ((DBG_DEBUG, DBG_TABBAR, "TXT_XOFF=%d, TXT_YOFF=%d, ATAB_EXTRA=%d, TAB_RADIUS=%d\n", TXT_XOFF, TXT_YOFF, ATAB_EXTRA, TAB_RADIUS));
 }
 
 
@@ -2350,7 +2340,7 @@ rxvt_tab_width (rxvt_t *r, const char *str)
 #endif
     {
 	int	    len;
-	RUINT16T    maxw = r->TermWin.maxTabWidth;
+	uint16_t    maxw = r->TermWin.maxTabWidth;
 
 	assert (str);
 	len = STRLEN (str);
@@ -2569,6 +2559,31 @@ rxvt_tabbar_move_tab (rxvt_t* r, short newPage)
 	for( i = min( newPage, curPage ); i <= max( newPage, curPage ); i++ )
 	    refresh_tabbar_tab( r, i);
     }
+
+    if( ISSET_OPTION( r, Opt2_syncTabTitle ) )
+	sync_tab_title( r, ATAB(r) );
+}
+
+/*
+ * Synchronize the window title to the title of the tab "page".
+ */
+void
+sync_tab_title( rxvt_t *r, int page )
+{
+    char wintitle[MAX_TAB_TXT];
+
+    if(
+	 IS_NULL( PVTS(r,page)->winTitleFormat )	||
+	 rxvt_percent_interpolate( r, page,
+	     PVTS(r,page)->winTitleFormat, STRLEN(PVTS(r,page)->winTitleFormat),
+	     wintitle, MAX_TAB_TXT ) <= 1
+      )
+    {
+	/* % interpolation failed / not possible */
+	rxvt_set_term_title( r, (unsigned char*) PVTS(r, page)->tab_title );
+    }
+    else
+	rxvt_set_term_title( r, (unsigned char*) wintitle );
 }
 
 /*----------------------- end-of-file (C source) -----------------------*/
