@@ -29,20 +29,6 @@
 #include "rxvt.h"
 
 
-#ifdef DEBUG_VERBOSE
-#define DEBUG_LEVEL 1
-#else 
-#define DEBUG_LEVEL 0
-#endif
-
-#if DEBUG_LEVEL
-#define DBG_MSG(d,x) if(d <= DEBUG_LEVEL) fprintf x
-#else
-#define DBG_MSG(d,x)
-#endif
-
-
-
 /*--------------------------------------------------------------------*
  *         BEGIN `INTERNAL' ROUTINE PROTOTYPES                        *
  *--------------------------------------------------------------------*/
@@ -102,7 +88,7 @@ static const char *const xnames[3] = {
     (sizeof(optList) / sizeof(optList[0]))
 
 static const struct {
-    const RUINT32T  flag;	/* Option flag */
+    const uint32_t  flag;	/* Option flag */
     const int	    doff;	/* data offset */
     const char*	    kw;		/* keyword */
     const char*	    opt;	/* option */
@@ -127,16 +113,21 @@ static const struct {
 	    "background color for a tab", 1),
     STRG( Rs_cwd, "workingDirectory", "wd", "dir",
 	    "Working directory to open new tabs in", 1 ),
-    STRG(Rs_holdExit, "holdExit", "hold", "Always|Clean|NotClean|Never",
+    STRG(Rs_holdExit, "holdExit", "hold", "number",
 	    "Keep tab open after child exits, based on childs exit status", 1 ),
     STRG(Rs_holdExitTxt, "holdExitText", "het",
 	"string", "text to show while holding the terminal", 1),
     STRG(Rs_holdExitTtl, "holdExitTitle", "heT",
 	"string", "Tab title of exited tabs", 1),
+    STRG(Rs_titleFormat, "titleFormat", "tf",
+	"string", "Displayed tab title format", 1),
+    STRG( Rs_winTitleFormat, "winTitleFormat", "wtf", "string",
+	    "format of window title (when syncing to tab title)", 1 ),
 
     /* Global options */
     STRG(Rs_maxTabWidth, "maxTabWidth", "mtw", "number", "maximum (char) title width of all tabs", 0),
-    STRG(Rs_minVisibleTabs, "minVisibleTabs", "mvt", "number", "minimum # of tabs to keep visible (requires xftpfn)", 0),
+    STRG(Rs_minVisibleTabs, "minVisibleTabs", "mvt", "number",
+	    "minimum # of tabs to keep visible (requires xftpfn)", 0),
     BOOL( "highlightTabOnBell", "htb", Opt2_hlTabOnBell,
 	    "highlighting inactive tabs only when bell sounds" ),
     BOOL( "syncTabTitle", "stt", Opt2_syncTabTitle,
@@ -153,6 +144,14 @@ static const struct {
 	    "synchronizing icon name with tab title" ),
     BOOL( "veryBoldFont", "vbf", Opt2_veryBold,
 	    "showing color text with bold font" ),
+#ifndef NO_BRIGHTCOLOR
+    BOOL( "boldColors", NULL, Opt2_boldColors, NULL),	/* Show bold primary
+							   colors using bright
+							   colors */
+    BOOL( "veryBright", NULL, Opt_veryBright, NULL),	/* Show bold primary
+							   colors using bold
+							   font also */
+#endif
 
 
     STRG(Rs_container_window, NULL, "into", "WinID", "Parent window id", 0),
@@ -182,8 +181,6 @@ static const struct {
     RSTRG(Rs_bgRefreshInterval, "bgRefreshInterval", "ms", 0),
 #endif
 
-    RSTRG(Rs_focusDelay, "focusDelay", "ms", 0 ),
-
     STRG(Rs_fade, "fading", "fade", "%", "make colors x% darker when losing focus", 0),
 
 #ifdef HAVE_SCROLLBARS
@@ -206,8 +203,6 @@ static const struct {
 	    "scroll-on-tty-output inhibit" ),
     BOOL( "scrollTtyKeypress", "sk", Opt_scrollTtyKeypress,
 	    "scroll-on-keypress" ),
-    BOOL( "scrollWithBuffer", "sw", Opt_scrollWithBuffer,
-	    "scroll-with-buffer" ),
 
     STRG(Rs_opacity, "opacity", "o", "%",
 	"transluscent window (true transparent) opaque degree", 0),
@@ -234,9 +229,7 @@ static const struct {
     STRG(Rs_tabbarPixmap, "tabbarPixmap", "tbpixmap", "file[;geom]", "tabbar background image", 0),
     BOOL( "tabUsePixmap", "tupixmap", Opt_tabPixmap,
 	    "use tabbar background image for tabs" ),
-# if 0 /* App icon not yet implemented */
     STRG(Rs_appIcon, "appIcon", "ic", "file[;geom]", "application icon file", 0),
-#endif
 #endif	/* BACKGROUND_IMAGE */
 
     BOOL( "utmpInhibit", "ut", Opt_utmpInhibit,
@@ -554,11 +547,20 @@ static const struct {
 	"Initial number of tabs/terminals", 0),
     STRG( Rs_initProfiles, "initProfileList", "ip", "profile list",
 	"List of profiles to load on startup", 0 ),
+
+    STRG(Rs_debug_masks, "dbgMasks", "dmask", "string", 
+	"List of debug masks separated by coma", 0),
+    STRG(Rs_debug_level, "dbgLevel", "dlevel", "string", 
+	"Name of number of the debug level", 0),
+
+#ifdef USE_FIFO
+    BOOL( "useFifo", NULL, Opt_useFifo, NULL ),
+#endif/*USE_FIFO*/
     INFO("e", "command arg ...", "command to execute")
 };
 
 /* Previously set options */
-static RUINT32T	pSetOpts[ MAX_OPTION_ARRAY ] = { 0u, 0u, 0u, 0u };
+static uint32_t	pSetOpts[ MAX_OPTION_ARRAY ] = { 0u, 0u, 0u, 0u };
 
 #undef INFO
 #undef STRG
@@ -761,7 +763,7 @@ rxvt_usage(int type)
 		" [options] [-e command args]\n\n"
 		"where resources (long-options) include:\n");
 
-#ifdef DEBUG_VERBOSE
+#ifdef DEBUG
 	    /*
 	     * Print short options, long options and descriptions so we can
 	     * generate man page.
@@ -879,7 +881,7 @@ rxvt_get_options(rxvt_t *r, int argc, const char *const *argv)
     static const char On[3] = "ON", Off[4] = "OFF";
 
 
-    DBG_MSG( 2, (stderr, "rxvt_get_options()\n") );
+    rxvt_dbgmsg ((DBG_DEBUG, DBG_RESOURCE, "rxvt_get_options()\n"));
 
     for (i = 1; i < argc; i++)
     {
@@ -890,7 +892,7 @@ rxvt_get_options(rxvt_t *r, int argc, const char *const *argv)
 	opt = argv[i];
 	profileNum = 0;	/* initialize profileNum to 0 by default */
 
-	DBG_MSG(1, (stderr, "argv[%d] = %s: ", i, opt));
+	rxvt_msg (DBG_INFO, DBG_RESOURCE, "argv[%d] = %s: \n", i, opt);
 	if (*opt == '-')
 	{
 	    flag = On;
@@ -906,7 +908,7 @@ rxvt_get_options(rxvt_t *r, int argc, const char *const *argv)
 	else
 	{
 	    bad_option = 1;
-	    rxvt_print_error("bad option \"%s\"", opt);
+	    rxvt_msg (DBG_ERROR, DBG_RESOURCE, "bad option \"%s\"", opt);
 	    continue;
 	}
 
@@ -961,9 +963,7 @@ rxvt_get_options(rxvt_t *r, int argc, const char *const *argv)
 			    offset, opt, profileNum, optList[entry].opt );
 		    bufshort[sizeof(bufshort)-1] = '\0';
 
-		    DBG_MSG( 3, ( stderr,
-				"Matched profile=%d buflong=%s bufshort=%s\n",
-				profileNum, buflong, bufshort ) );
+		    rxvt_dbgmsg ((DBG_DEBUG, DBG_RESOURCE, "Matched profile=%d buflong=%s bufshort=%s\n", profileNum, buflong, bufshort ));
 		}
 
 		/* If no profile number is specified, use 0 by default */
@@ -977,9 +977,7 @@ rxvt_get_options(rxvt_t *r, int argc, const char *const *argv)
 
 		    profileNum = 0;
 
-		    DBG_MSG( 3, ( stderr,
-				"Matched default buflong=%s bufshort=%s\n",
-				buflong, bufshort ) );
+		    rxvt_dbgmsg ((DBG_DEBUG, DBG_RESOURCE, "Matched default buflong=%s bufshort=%s\n", buflong, bufshort ));
 		}
 	    }
 	    else if (optList[entry].kw)
@@ -1024,13 +1022,11 @@ rxvt_get_options(rxvt_t *r, int argc, const char *const *argv)
 	    {
 		const char   *str = argv[++i];
 
-		DBG_MSG(2, (stderr, "string (%s,%s) = ",
-		    optList[entry].opt ? optList[entry].opt : "nil",
-		    optList[entry].kw ? optList[entry].kw : "nil"));
+		rxvt_dbgmsg ((DBG_DEBUG, DBG_RESOURCE, "string (%s,%s) = ", optList[entry].opt ? optList[entry].opt : "nil", optList[entry].kw ? optList[entry].kw : "nil"));
 
 		if (flag == On && str && (optList[entry].doff != -1))
 		{
-		    DBG_MSG(2, (stderr, "\"%s\"\n", str));
+		    rxvt_dbgmsg ((DBG_DEBUG, DBG_RESOURCE, "\"%s\"\n", str));
 		    r->h->rs[optList[entry].doff + profileNum] = str;
 		    /*
 		     * special cases are handled in main.c:main() to allow X
@@ -1038,17 +1034,14 @@ rxvt_get_options(rxvt_t *r, int argc, const char *const *argv)
 		     * default values
 		     */
 		}
-#ifdef DEBUG_VERBOSE
 		else
-		    DBG_MSG(2, (stderr, "???\n"));
-#endif
+		    rxvt_dbgmsg ((DBG_DEBUG, DBG_RESOURCE, "???\n"));
 	    }
 
 	    /* boolean value */
 	    else
 	    {
-		DBG_MSG( 2, ( stderr, "boolean (%s,%s) = %s\n",
-		    optList[entry].opt, optList[entry].kw, flag ) );
+		rxvt_dbgmsg ((DBG_DEBUG, DBG_RESOURCE, "boolean (%s,%s) = %s\n", optList[entry].opt, optList[entry].kw, flag ));
 
 		if( flag == On )
 		    SET_OPTION( r, optList[entry].flag );
@@ -1098,7 +1091,7 @@ rxvt_get_options(rxvt_t *r, int argc, const char *const *argv)
 		else
 		    bad_option = 1;
 
-		rxvt_print_error("%s option \"%s\"", msg, --opt);
+		rxvt_msg (DBG_ERROR, DBG_RESOURCE, "%s option \"%s\"", msg, --opt);
 	    }
 	}
     }
@@ -1110,11 +1103,11 @@ rxvt_get_options(rxvt_t *r, int argc, const char *const *argv)
 	rxvt_usage(0);
 #endif
 
-	rxvt_print_error( "Use -h, -help or --help to get help" );
+	rxvt_msg (DBG_ERROR, DBG_RESOURCE,  "Use -h, -help or --help to get help" );
 	exit( EXIT_FAILURE );
     }
 
-    DBG_MSG( 2, (stderr, "rxvt_get_options() done.\n") );
+    rxvt_dbgmsg ((DBG_DEBUG, DBG_RESOURCE, "rxvt_get_options() done.\n"));
 }
 
 
@@ -1137,7 +1130,7 @@ rxvt_get_xdefaults(rxvt_t *r, FILE *stream, const char *name)
     Bool    noReplace = !STRCMP( name, APL_CLASS )
 			    || !STRCMP( name, APL_SUBCLASS);
 
-    DBG_MSG(1, (stderr, "rxvt_get_xdefaults (%s)\n", name));
+    rxvt_msg (DBG_INFO, DBG_RESOURCE, "rxvt_get_xdefaults (%s)\n", name);
 
     if (IS_NULL(stream))
 	return;
@@ -1214,8 +1207,7 @@ rxvt_get_xdefaults(rxvt_t *r, FILE *stream, const char *name)
 			STRNCPY (kw, buf, sizeof(kw)-1);
 			kw[sizeof(kw)-1] = '\0';
 
-			DBG_MSG( 3, ( stderr, "Matched profile=%d kw=%s\n",
-				profileNum, kw) );
+			rxvt_dbgmsg ((DBG_DEBUG, DBG_RESOURCE, "Matched profile=%d kw=%s\n", profileNum, kw));
 		    }
 
 		    /*
@@ -1226,9 +1218,7 @@ rxvt_get_xdefaults(rxvt_t *r, FILE *stream, const char *name)
 		    {
 			profileNum = 0;
 
-			DBG_MSG( 3, ( stderr,
-				    "Matched default kw=%s for option %s",
-				    kw, str) );
+			rxvt_dbgmsg ((DBG_DEBUG, DBG_RESOURCE, "Matched default kw=%s for option %s", kw, str));
 		    }
 		}
 
@@ -1440,7 +1430,7 @@ rxvt_extract_resources (
 	{
 	    if( r->h->rs[i] == emptyResource )
 	    {
-		DBG_TMSG( 3, ( stderr, "Setting resource #%d to NULL\n", i ) );
+		rxvt_dbgmsg ((DBG_DEBUG, DBG_RESOURCE, "Setting resource #%d to NULL\n", i));
 		r->h->rs[i] = NULL;
 	    }
 	}
@@ -1477,7 +1467,14 @@ rxvt_extract_resources (
     if (r->h->rs[Rs_multichar_encoding])
 	rxvt_set_multichar_encoding(r, r->h->rs[Rs_multichar_encoding]);
     else
-	rxvt_set_multichar_encoding(r, MULTICHAR_ENCODING);
+    {
+	char*    enc;
+
+	if (NOT_NULL(enc = rxvt_get_encoding_from_locale (r)))
+	    rxvt_set_multichar_encoding (r, enc);
+	else
+	    rxvt_set_multichar_encoding(r, MULTICHAR_ENCODING);
+    }
 #endif
 
 #ifdef GREEK_SUPPORT
